# yacon.models.users.py
import logging, copy

from django.db import models
from django.db import connection
from django.db.utils import IntegrityError
from django.contrib.auth.models import User

from yacon.models.common import TimeTrackedModel

logger = logging.getLogger(__name__)

# ============================================================================

class UsernameError(Exception):
    pass


class UserProfileBase(TimeTrackedModel):
    user = models.OneToOneField(User)

    class Meta:
        app_label = 'yacon'
        abstract = True

    @classmethod
    def create(cls, username, first_name, last_name, email, password, **kwargs):
        try:
            user = User.objects.create_user(username, email, password)
            user.first_name = first_name
            user.last_name = last_name

            if 'is_active' in kwargs:
                user.is_active = kwargs.pop('is_active')
            if 'is_staff' in kwargs:
                user.is_staff = kwargs.pop('is_staff')
            if 'is_superuser' in kwargs:
                user.is_superuser = kwargs.pop('is_superuser')

            user.save()
        except IntegrityError:
            # rollback the attempted commit and throw our own error
            connection._rollback()
            err = UsernameError('username %s existed already' % username)
            raise err

        profile = cls(user=user, **kwargs)
        profile.save()
        return profile

    @classmethod
    def create_from_data(cls, data):
        kwargs = copy.copy(data)
        username = kwargs.pop('username')
        first_name = kwargs.pop('first_name')
        last_name = kwargs.pop('last_name')
        email = kwargs.pop('email')
        password1 = kwargs.pop('password1')
        kwargs.pop('password2')

        return cls.create(username, first_name, last_name, email, password1,
            **kwargs)

    def update_profile(self, data):
        """Updates the UserProfile and corresponding User object based on the
        data that was passed in.  Note: do NOT attempt a password change this
        way."""
        user_changed = False
        profile_changed = False

        for key, value in data.items():
            if hasattr(self.user, key):
                setattr(self.user, key, value)
                user_changed = True
            elif hasattr(self, key):
                setattr(self, key, value)
                profile_changed = True
            else:
                logger.error('neither profile or profile.user has key "%s"',
                    key)

        try:
            if user_changed:
                self.user.save()
        except IntegrityError:
            connection._rollback()
            raise UsernameError('username %s already existed' %
                data['username'])

        if profile_changed:
            self.save()

    def permission_to_create_page(self, page_type, node):
        """Returns True if this user is allowed to create the page of the
        given type at the given uri.  Base implementation only returns True if
        the user is a superuser.

        :param page_type: PageType object of the page about to be created
        :param node: Node object where the page is being placed

        :returns: Boolean indicating if the user has permission to create a
            page of that type in that location
        """
        return self.user.is_superuser

    def permission_to_edit_page(self, page, context={}):
        """Returns True if this user is allowed to edit the given page.  Base
        implementation returns True if the user is the owner or a superuser.
        """
        if self.user.is_superuser or self.user == page.metapage.owner:
            return True

        return False


class UserProfile(UserProfileBase):
    class Meta:
        app_label = 'yacon'



from django.core.paginator import Paginator, Page, PageNotAnInteger, EmptyPage

class InfinitePaginator(Paginator):
    """
    Paginator designed for cases when it's not important to know how many total
    pages.  This is useful for any object_list that has no count() method or can
    be used to improve performance for MySQL by removing counts.

    The orphans parameter has been removed for simplicity and there's a link
    template string for creating the links to the next and previous pages.
    """

    def __init__(self, object_list, per_page, allow_empty_first_page=True,
        link_template='/page/%d/'):
        orphans = 0 # no orphans
        super(InfinitePaginator, self).__init__(object_list, per_page, orphans,
            allow_empty_first_page)
        # no count or num pages
        del self._num_pages, self._count
        # bonus links
        self.link_template = link_template

    def validate_number(self, number):
        """
        Validates the given 1-based page number.
        """
        try:
            number = int(number)
        except ValueError:
            raise PageNotAnInteger('That page number is not an integer')
        if number < 1:
            raise EmptyPage('That page number is less than 1')
        return number

    def page(self, number):
        """
        Returns a Page object for the given 1-based page number.
        """
        number = self.validate_number(number)
        bottom = (number - 1) * self.per_page
        top = bottom + self.per_page
        page_items = self.object_list[bottom:top]
        # check moved from validate_number
        if not page_items:
            if number == 1 and self.allow_empty_first_page:
                pass
            else:
                raise EmptyPage('That page contains no results')
        return InfinitePage(page_items, number, self)

    def _get_count(self):
        """
        Returns the total number of objects, across all pages.
        """
        raise NotImplementedError
    count = property(_get_count)

    def _get_num_pages(self):
        """
        Returns the total number of pages.
        """
        raise NotImplementedError
    num_pages = property(_get_num_pages)

    def _get_page_range(self):
        """
        Returns a 1-based range of pages for iterating through within
        a template for loop.
        """
        raise NotImplementedError
    page_range = property(_get_page_range)


class InfinitePage(Page):

    def __repr__(self):
        return '<Page %s>' % self.number

    def has_next(self):
        """
        Checks for one more item than last on this page.
        """
        try:
            self.paginator.object_list[self.number * self.paginator.per_page]
        except IndexError:
            return False
        return True

    def end_index(self):
        """
        Returns the 1-based index of the last object on this page,
        relative to total objects found (hits).
        """
        return ((self.number - 1) * self.paginator.per_page +
            len(self.object_list))

    #Bonus methods for creating links

    def next_link(self):
        if self.has_next():
            return self.paginator.link_template % (self.number + 1)
        return None

    def previous_link(self):
        if self.has_previous():
            return self.paginator.link_template % (self.number - 1)
        return None

class FinitePaginator(InfinitePaginator):
    """
    Paginator for cases when the list of items is already finite.

    A good example is a list generated from an API call. This is a subclass
    of InfinitePaginator because we have no idea how many items exist in the
    full collection.

    To accurately determine if the next page exists, a FinitePaginator MUST be
    created with an object_list_plus that may contain more items than the
    per_page count.  Typically, you'll have an object_list_plus with one extra
    item (if there's a next page).  You'll also need to supply the offset from
    the full collection in order to get the page start_index.

    This is a very silly class but useful if you love the Django pagination
    conventions.
    """

    def __init__(self, object_list_plus, per_page, offset=None,
        allow_empty_first_page=True, link_template='/page/%d/'):
        super(FinitePaginator, self).__init__(object_list_plus, per_page,
            allow_empty_first_page, link_template)
        self.offset = offset

    def validate_number(self, number):
        super(FinitePaginator, self).validate_number(number)
        # check for an empty list to see if the page exists
        if not self.object_list:
            if number == 1 and self.allow_empty_first_page:
                pass
            else:
                raise EmptyPage('That page contains no results')
        return number

    def page(self, number):
        """
        Returns a Page object for the given 1-based page number.
        """
        number = self.validate_number(number)
        # remove the extra item(s) when creating the page
        page_items = self.object_list[:self.per_page]
        return FinitePage(page_items, number, self)

class FinitePage(InfinitePage):

    def has_next(self):
        """
        Checks for one more item than last on this page.
        """
        try:
            self.paginator.object_list[self.paginator.per_page]
        except IndexError:
            return False
        return True

    def start_index(self):
        """
        Returns the 1-based index of the first object on this page,
        relative to total objects in the paginator.
        """
        ## TODO should this holler if you haven't defined the offset?
        return self.paginator.offset



# yacon.helpers.py
import logging

from yacon import conf
from yacon.models.site import Site
from yacon.models.pages import PageType, BlockType
from yacon.utils import get_profile

logger = logging.getLogger(__name__)

# ============================================================================
# Page and Block Creation Helpers
# ============================================================================

def create_page_type(name, template, block_types=[]):
    """Creates and saves a new PageType object

    :param name: name of the PageType
    :param template: template to associate with the PageType
    :param block_types: list of BlockType objects used in the template

    :returns: the created PageType object
    """
    pt = PageType(name=name, template=template)
    pt.save()
    for block_type in block_types:
        pt.block_types.add(block_type)
    return pt


def create_dynamic_page_type(name, function_name):
    """Creates and saves a new PageType using a dynamic rendering function

    :param name: name of the PageType
    :param function_name: name of function and the module it is in to call to
        dynamically generate content for this page. Uses dot notation, example
        "module.sub.function" would do "from module.sub import function" and 
        then call "function()"

    :returns: the created PageType object
    """
    pt = PageType.objects.create(name=name, dynamic=function_name)
    return pt


def create_block_type(name, key, module_name='yacon.models.content', 
        content_handler_name='FlatContent', content_handler_parms={}):
    """Creates and saves a new BlockType object

    :param name: name of the BlockType
    :param key: identifying key of the BlockType
    :param mod: string specifying the module the ContentHandler class is in 
    :param content_handler: string specifying the name of the ContentHandler
        class for this BlockType
    :param content_handler_parms: optional dictionary of parameters to
        initialize the ContentHandler with

    :returns: the created BlockType object
    """
    bt = BlockType(name=name, key=key, module_name=module_name, 
        content_handler_name=content_handler_name, 
        content_handler_parms=content_handler_parms)
    bt.save()
    return bt

# ============================================================================
# View Helpers
# ============================================================================

def prepare_context(request, uri=None):
    """Creates the base context for rendering a page, calls the custom page
    context function if defined.
    """
    if not uri:
        uri = request.get_full_path()
    
    data = {
        'site':Site.get_site(request),
        'uri':uri,
    }

    page_context = conf.custom.page_context
    page_context(request, uri, data)

    return data


def permission_to_edit_page(request, page, context={}):
    """Calls user.permission_to_edit_page() on the user in the request, or
    returns False if no user in the request."""
    if hasattr(request, 'user') and request.user.is_authenticated():
        profile = get_profile(request.user)
        if profile:
            return profile.permission_to_edit_page(page, context)

    return False



# yacon.models.hierarchy.py
import re, exceptions, logging

from django.db import models
from django.template.defaultfilters import slugify

from treebeard.mp_tree import MP_Node

from yacon.models.common import Language, TimeTrackedModel, NodePermissionTypes
from yacon.models.pages import Page, MetaPage
from yacon.definitions import SLUG_LENGTH

logger = logging.getLogger(__name__)

MATCH_WORD = re.compile('^[-\w]*$')

# ============================================================================
# Exceptions
# ============================================================================

class BadSlug(exceptions.Exception):
    pass

# ============================================================================
# Site and Page Hierarchy Management
# ============================================================================

class BaseNode(MP_Node, TimeTrackedModel):
    class Meta:
        abstract = True
        app_label = 'yacon'

    def has_children(self):
        return self.get_children_count() > 0


class Node(BaseNode):
    """A Site object represents a collection of page hiearchies and menus that
    are represented as a series of trees.  The Node object is a single node in
    one of those trees.

    Each node is identified by a slug with a series of slugs forming a path to
    an individual Node.  Slugs can be translated into multiple languages, thus
    allowing for multiple paths identifying a single unique Node.  A path
    should never point to two different Nodes in a single Site but as this is
    computationally expensive to enforce it is left to the developer to
    ensure.

    It is highly suggested that the factory methods be used to construct new
    Node objects.  There are two types of factory methods: those for creating
    root objects are on the Site class, those for creating children are on the
    Node class itself.  Node objects are based on django-treebeard's MP_Node
    object and any of those methods are available, but the factory methods
    attempt to abstract some of the associated complexity and enforce rules
    around formatting of slugs, etc.  
    """
    site = models.ForeignKey('yacon.Site')
    permission = models.CharField(max_length=3, choices=NodePermissionTypes,
        default=NodePermissionTypes.INHERIT)

    class Meta:
        app_label = 'yacon'

    def __unicode__(self):
        return 'Node: %s (%s)' % (self.name, self.slug)

    # -----------------------------------------------------------------------
    # Utility Methods
    def validate_slug(self, slug, auto_fix=False):
        """Raises a BadSlug exception if "slug" is not valid."""
        if len(slug) > SLUG_LENGTH:
            if not auto_fix:
                raise BadSlug('Max slug length is %d characters' % SLUG_LENGTH)

            # strip the slug to max length
            slug = slug[:SLUG_LENGTH]

        if not MATCH_WORD.search(slug):
            if not auto_fix:
                raise BadSlug('Slug must be of form [0-9a-zA-Z_-]*')

            # slugify the slug
            slug = slugify(slug)

        # find the slugs of all the children of this node as well as all the
        # pages in the node
        existing = NodeTranslation.objects.filter(
            node__in=self.get_children()).values_list('slug', flat=True)
        if slug in existing and not auto_fix:
            raise BadSlug('A child node already has the given slug')

        existing = list(existing)
        metapages = MetaPage.objects.filter(node=self)
        slugs = Page.objects.filter(metapage__in=metapages).values_list(
            'slug', flat=True)
        existing.extend(slugs)

        aliased_metapages = MetaPage.objects.filter(alias=self)
        slugs = Page.objects.filter(metapage__in=aliased_metapages).values_list(
            'slug', flat=True)
        existing.extend(slugs)

        if slug in existing and not auto_fix:
            raise BadSlug('A page in this node already has the given slug.')

        if slug not in existing:
            return slug

        # attempt to auto_fix the slug
        suffix = ''
        i = 2
        while(True):
            new_slug = slug + suffix
            if new_slug not in existing:
                return new_slug

            suffix = '-%d' % i
            i += 1

    # -----------------------------------------------------------------------
    # Factory/Fetch Methods
    def create_child(self, name, slug, translations={},
            permission=NodePermissionTypes.INHERIT):
        """Creates a Node object as a child of this Node.  Name and slug for
        the default language are passed in.  An optional dictionary of
        Language objects mapped to name/slug tuples can be used to
        populate other translations.

        :param name: name of Node in default language
        :param slug: slug for Node in default language
        :param translations: dictionary mapping language codes to tuples of
            name/slug pairs to be used to populate translations.  Example:
            {Language('en-GB'):('Colour','colour'), Language('fr'):('Couleur', 
            'couleur'), }

        :returns: newly created child Node

        :raises: BadSlug, if the slug contains any non-alpha-numeric
            character, exceeds 25 characters in length, or exists at this
            level already
        """
        translations[self.site.default_language] = (name, slug)

        # check for bad slugs
        for key, value in translations.items():
            (name, slug) = value
            self.validate_slug(slug)

        # no bad slugs, create the child node
        child = self.add_child(site=self.site, permission=permission)

        # add translations to child
        for key, value in translations.items():
            (name, slug) = value

            tx = NodeTranslation(node=child, language=key, name=name, slug=slug)
            tx.save()

        return child

    # -----------------------------------------------------------------------
    # Getter Methods
    @property 
    def name(self):
        """Returns the name for this Node in the Site's default translation"""
        return self.get_name()

    @property 
    def slug(self):
        """Returns the slug for this Node in the Site's default translation"""
        return self.get_slug()

    @property
    def has_missing_translations(self):
        """Returns True if there are languages in the site that there are no
        translations for in this node."""
        return self.nodetranslation_set.count() != self.site.language_count()

    @property
    def default_metapage(self):
        mps = self.metapage_set.filter(is_node_default=True)
        if len(mps) == 0:
            return None

        return mps[0]

    def get_default_page(self, language):
        """If this Node has an associated default MetaPage item return a
        translated Page for it in the given Language.  

        :param language: Language object to use for getting the Page, can be
            None to get the default language

        :returns: Page representing the default page for this Node, or None if
            there isn't one, or isn't one in the given language
        """
        if self.default_metapage == None:
            return None

        if language == None:
            # no language passed in, use default language
            return self.default_metapage.get_default_translation()

        return self.default_metapage.get_translation(language)

    def get_name(self, language=None):
        """Returns the name for this Node in the given Language.  If no
        Language is passed in then the Site's default Language is used.  

        :param langauge: optional parameter specifying the Language to return
            the Node's name in.  If not given the Site's default Language is
            used

        :returns: string containing desired Node name
        """
        if language == None:
            language = self.site.default_language

        try:
            tx = NodeTranslation.objects.get(node=self, language=language)
            return tx.name
        except NodeTranslation.DoesNotExist:
            return None

    def get_slug(self, language=None):
        """Returns the slug for this Node in the given Language.  If no
        Language is passed in then the Site's default language is used.  

        :param langauge: optional parameter specifying the Language to return
            the Node's slug in.  If not given the Site's default Language is
            used

        :returns: string containing desired Node slug
        """
        if language == None:
            language = self.site.default_language

        try:
            tx = NodeTranslation.objects.get(node=self, language=language)
            return tx.slug
        except NodeTranslation.DoesNotExist:
            return None

    def has_slug(self, find_slug):
        """Returns true if one of the NodeTranslation for this Node contains
        the given slug.

        :param find_slug: slug to search for

        :returns: True if find_slug matches one of the slug translations
        """

        txs = NodeTranslation.objects.filter(node=self, slug=find_slug)
        return len(txs) > 0

    def language_of_slug(self, find_slug):
        """Returns the Language object in the NodeTranslation object that
        contains the given slug

        :param find_slug: slug to find the Language for

        :returns: Language object
        """
        tx = NodeTranslation.objects.get(node=self, slug=find_slug)
        return tx.language

    @property
    def effective_permission(self):
        """Returns the effective permission value of this Node, one of either
        the node's permission attribute, or if the attribute is INHERIT, it
        returns the inherited value.
        
        :returns: NodePermissionType Enum value
        """
        if self.permission != NodePermissionTypes.INHERIT:
            return self.permission

        # our permission value is inherit, need to determine what we're
        # inheriting
        if self.is_root():
            return NodePermissionTypes.PUBLIC

        node = self.get_parent()
        while(node):
            if node.permission != NodePermissionTypes.INHERIT:
                return node.permission

            if node.is_root():
                return NodePermissionTypes.PUBLIC

            node = node.get_parent()

    @property
    def permission_string(self):
        return NodePermissionTypes.get_value(self.permission)

    @property
    def effective_permission_string(self):
        return NodePermissionTypes.get_value(self.effective_permission)

    # -----------------------------------------------------------------------
    # Tree Walking Methods

    def _walk_tree_to_string(self, node, output, indent):
        """Breadth first walk of tree returning node as string

        :param node -- node to walk
        :param string -- string to append to before returning
        :param output -- list of lines containing a string from each node
            visited
        :param indent -- how much to indent the displayed node
        """
        output.append('%s%s (%s)' % (3*indent*' ', node.name, node.slug))
        for child in node.get_children():
            self._walk_tree_to_string(child, output, indent+1)

    def tree_to_string(self):
        """Returns a string representation of the sub-tree using the 'self' 
        node as root"""

        output = []
        self._walk_tree_to_string(self, output, 0)
        return "\n".join(output)

    def node_to_path(self, language=None):
        """Returns a path string for this node

        :param language: optional parameter specifying the Language to express
            the path in.  If none specified then Site object's default
            Language is used
        """
        if self.is_root():
            return '/'

        nodes = []
        for node in self.get_ancestors():
            nodes.append(node)

        # get_ancestors() will include root which we don't use in paths and
        # won't include us, so remove the first and add this node at the end
        nodes.pop(0)
        nodes.append(self)
        path = '/'
        for node in nodes:
            slug = node.get_slug(language)
            if slug == None:
                return None

            path += slug + '/'
        return path


class NodeTranslation(TimeTrackedModel):
    """Stores translations of Node names and slugs according to Language
    object.  
    """
    language = models.ForeignKey(Language, related_name='+')
    name = models.CharField(max_length=30)
    slug = models.CharField(max_length=SLUG_LENGTH)
    node = models.ForeignKey(Node)

    class Meta:
        app_label = 'yacon'

    def get_path(self):
        return self.node.node_to_path(self.language)

# ============================================================================
# Menu Management
# ============================================================================

class MenuItem(BaseNode):
    """A MenuItem is a node in a hierarchy that is displayed to the user,
    typically for navigation, that is independent of the document hierarchy.
    A MenuItem can be "clickable" and associated with a Metapage, or not
    "clickable" and simply a placeholder in the hierarchy.  

    A Metapage is only allowed to be mapped to a single MenuItem across all
    menus.  This allows for easy determination from the URL as to where in the
    menu the user is.  If you want the same content to show up in two
    MenuItems then create a Metapage alias.
    """
    metapage = models.OneToOneField('yacon.MetaPage', blank=True, null=True,
        unique=True)
    link = models.TextField(blank=True)
    menu = models.ForeignKey('yacon.Menu')
    requires_login = models.BooleanField(default=False)
    requires_admin = models.BooleanField(default=False)

    class Meta:
        app_label = 'yacon'

    def __unicode__(self):
        return 'MenuItem(id=%s)' % self.id

    def create_child(self, metapage=None, link='', translations={}, 
            requires_login=False, requires_admin=False):
        child = self.add_child(metapage=metapage, link=link, menu=self.menu,
            requires_login=requires_login, requires_admin=requires_admin)
        for key, value in translations.items():
            MenuItemTranslation.objects.create(language=key, name=value, 
                menuitem=child)
        return child

    # --------------------
    # Translation Access Methods
    def get_translation(self, language):
        """Returns a MenuItemTranslation object for this MenuItem in the given 
        language.

        :param language: Language for the corresponding translation

        :returns: MenuItemTranslation object or None
        """
        try:
            return MenuItemTranslation.objects.get(menuitem=self, 
                language=language)
        except MenuItemTranslation.DoesNotExist:
            return None

    def get_translations(self, ignore_default=False):
        """Returns a list of MenuItemTranslation objects representing the 
        for this MenuItem.

        :param ignore_default: if True the default language will not be
            returned in the list; defaults to False

        :returns: list of MenuItemTranslation objects
        """
        try:
            txs = MenuItemTranslation.objects.filter(menuitem=self)
            if ignore_default:
                txs = txs.exclude(language=self.menu.site.default_language)

            return txs

        except MenuItemTranslation.DoesNotExist:
            return None

    def get_default_translation(self):
        """Returns the MenuItemTranslation object for the site default language

        :returns: MenuItemTranslation object or None
        """
        try:
            return MenuItemTranslation.objects.get(menuitem=self, 
                language=self.menu.site.default_language)
        except MenuItemTranslation.DoesNotExist:
            return None

    @property
    def has_missing_translations(self):
        """Returns True if there are languages in the site that there are no
        translations for in this metapage."""
        txs = self.menuitemtranslation_set.count()
        langs = self.menu.site.language_count()

        return txs != langs

    @property
    def can_move_out(self):
        """Returns True if this item is not at root depth."""
        depth = self.get_depth() 
        return depth != 1

    @property
    def can_move_up(self):
        """Returns True if this item is not the first sibling."""
        first = self.get_first_sibling() 
        return first != self

    @property
    def can_move_down(self):
        """Returns True if this item is not the last sibling."""
        last = self.get_last_sibling() 
        return last != self


class Menu(TimeTrackedModel):
    """Represents a menu associated with a site.  The name in this menu is for
    configuration purposes only, the MenuItems contained within it (and their
    translations) are what are displayed to the user.
    """
    name = models.CharField(max_length=30)
    site = models.ForeignKey('yacon.Site')

    @property
    def first_level(self):
        return MenuItem.objects.filter(menu=self, depth=1)

    def create_child(self, metapage=None, link='', translations={}, 
            requires_login=False, requires_admin=False):
        item = MenuItem.add_root(metapage=metapage, link=link, menu=self,
            requires_login=requires_login, requires_admin=requires_admin)
        for key, value in translations.items():
            MenuItemTranslation.objects.create(language=key, name=value, 
                menuitem=item)
        return item


class MenuItemTranslation(TimeTrackedModel):
    """Stores translations of MenuItem names according to Language object.  
    """
    language = models.ForeignKey(Language, related_name='+')
    name = models.CharField(max_length=30)
    menuitem = models.ForeignKey(MenuItem)

    class Meta:
        app_label = 'yacon'

    def __unicode__(self):
        return 'MenuItemTranslation: %s (%s)' % (self.name, self.language.code)

    @property 
    def page(self):
        """Returns the Page that corresponds to this language for the MetaPage
        that this item's menuitem points to.  Essentially, where this item
        points."""
        if hasattr(self, '_cached_page'):
            return self._cached_page

        self._cached_page = self.menuitem.metapage.get_translation(
            self.language)
        return self._cached_page



# yacon.views.users_panel.py
#
# Methods for the Users Panel
#

import logging

from django.contrib.auth import login
from django.contrib.auth.forms import AdminPasswordChangeForm
from django.forms.util import ErrorList
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404

from yacon import conf
from yacon.decorators import superuser_required
from yacon.models.users import UsernameError

logger = logging.getLogger(__name__)
USER_CURATOR = conf.custom.user_curator

# ============================================================================

@superuser_required
def list_users(request):
    sort_by = request.GET.get('sort', 'user__username')
    direction = request.GET.get('direction')

    sort_args = filter(None, sort_by.split(','))
    if direction == 'rev':
        sort_args = ['-%s' % arg for arg in sort_args]

    profiles = USER_CURATOR.profile_class.objects.all().order_by(*sort_args)
    data = {
        'title':'User Listing',
        'profiles':profiles,
        'default_sort':'user__username',
    }

    return render_to_response('yacon/nexus/list_users.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def edit_user(request, profile_id):
    profile = get_object_or_404(USER_CURATOR.profile_class, id=profile_id)

    if request.method == 'POST':
        form = USER_CURATOR.update_form_class(request.POST)
        if form.is_valid():
            try:
                profile.update_profile(form.cleaned_data)
                return HttpResponseRedirect('/yacon/nexus/users/list_users/')
            except UsernameError:
                errors = form._errors.setdefault('username', ErrorList())
                errors.append('Username already exists')
    else: # GET
        form = USER_CURATOR.update_form_class(user=profile.user, 
            profile=profile)

    data = {
        'title':'Edit User',
        'profile':profile,
        'form':form,
    }

    return render_to_response('yacon/nexus/edit_user.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def add_user(request):
    data = {
        'title':'Add User',
        'clear_autocomplete':True,
    }
    if request.method == 'POST':
        form = USER_CURATOR.add_form_class(request.POST)
        if form.is_valid():
            try:
                USER_CURATOR.profile_class.create_from_data(form.cleaned_data)
                return HttpResponseRedirect('/yacon/nexus/users/list_users/')
            except UsernameError:
                errors = form._errors.setdefault('username', ErrorList())
                errors.append('Username already exists')

        # form didn't validate, turn off clear
        data['clear_autocomplete'] = False
    else: # GET
        form = USER_CURATOR.add_form_class()

    data['form'] = form
    return render_to_response('yacon/nexus/edit_user.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def user_password(request, profile_id):
    profile = get_object_or_404(USER_CURATOR.profile_class, id=profile_id)

    if request.method == 'POST':
        form = AdminPasswordChangeForm(profile.user, request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/yacon/nexus/users/list_users/')
    else: # GET
        form = AdminPasswordChangeForm(profile.user)

    data = {
        'title':'Change User Password',
        'profile':profile,
        'form':form,
        'clear_autocomplete':True,
    }

    return render_to_response('yacon/nexus/edit_user.html', data, 
        context_instance=RequestContext(request))


@superuser_required
def switch_to_user(request, profile_id):
    profile = get_object_or_404(USER_CURATOR.profile_class, id=profile_id)
    user = profile.user
    user.backend = 'django.contrib.auth.backends.ModelBackend'
    login(request, user)

    return HttpResponseRedirect('/')



# vim: set fileencoding=utf-8 :
#
# yacon_create_test_data.py
# blame ctrudeau chr(64) arsensa.com
#
# Unit tests for Yacon

from django.core.management.base import BaseCommand
from django.core import management

from yacon.models.common import Language
from yacon.models.site import Site
from yacon.models.pages import MetaPage, Translation
from yacon.models.hierarchy import NodeTranslation, Menu
from yacon.models.users import UserProfile
from yacon.helpers import create_page_type, create_block_type

class Command(BaseCommand):
    def handle(self, *args, **options):
        # this management command is used to create test data for yacon

        # if it hasn't been done already, use the yacon_create_defaults script
        # as the basis for our data
        try:
            # get the default site
            site = Site.objects.get(name='Localhost Site')
        except Site.DoesNotExist:
            # default site creation script wasn't run, do it ourselves
            management.call_command('yacon_create_defaults')
            site = Site.objects.get(name='Localhost Site')

        # fetch default language
        english = site.default_language

        # add another language to the Site
        french = Language.factory(name='French', identifier='fr')
        site.alternate_language.add(french)

        # create content hierarchy tree
        articles = site.doc_root.create_child('Articles', 'articles', {\
            french:("L'Article", 'lesarticles')})
        health = articles.create_child('Health', 'health', {\
            french:("La Sante", 'sante')})
        fitness = articles.create_child('Fitness', 'fitness', {\
            french:("De Fitness", 'defitness')})
        money = articles.create_child('Money', 'money', {\
            french:("L'Argent", 'argent')})
        blog = site.doc_root.create_child('Blogs', 'blogs', {\
            french:("Le Blog", 'leblog')})

        # so we can test what happens if a node is missing a default language,
        # find the english money node translation and remove it
        tx = NodeTranslation.objects.get(node=money, language=english)
        tx.delete()

        # create templates for our content
        bt_user = create_block_type('User Content', 'blurb', 
            'yacon.models.content', 'FlatContent')
        bt_poll = create_block_type('Poll Content', 'poll', 
            'yacon.models.content', 'FlatContent')
        bt_blog = create_block_type('Blog Content', 'blog', 
            'yacon.models.content', 'FlatContent')
        create_block_type('Right Now Content', 'right_now', 
            'yacon.models.content', 'DynamicContent', 
            {'module':'yacon.examples.dynamic', 'function':'right_now'})

        pt_article = create_page_type('Article Type', 'examples/article.html',
            [bt_user, bt_poll])
        pt_blog = create_page_type('Blog Type', 'examples/blog.html', [bt_blog])

        # create a couple of menus
        menu1 = Menu.objects.create(name='Menu 1', site=site)

        # -----------------
        # create some pages
        mp = MetaPage.create_translated_page(site.doc_root, pt_article, [
            Translation(english,
                'Home Page', 'homepage',
                {
                    bt_user:('<p>This is a most excellent home page.</p>'),
                    bt_poll:"""
<h3>Poll: Is This Your Favourite Home Page?</h3>
<ul>
<li>Yes</li>
<li>Yes, most definitely</li>
</ul>
""",
                }),
            ]
        )
        mp.make_default_for_node()
        site.doc_root.save()
        menu1.create_child(mp)
        sub_menu = menu1.create_child(None, { english:'Sub Menu1' })
        sm2 = sub_menu.create_child(None, { english:'Sub Menu1.1' })
        sm2.create_child(None, { english:'Sub Menu1.1.1' })

        mp = MetaPage.create_translated_page(health, pt_article, [
            Translation(english,
                'Steak is good', 'steak',
                {
                    bt_user:('<p>Steak should be as good for you as it '
                        'tastes.</p>'),
                    bt_poll:"""
<h3>Poll: Favourite Steak?</h3>
<ul>
<li>T-Bone</li>
<li>Filet Mignon</li>
<li>Porterhouse</li>
</ul>
""",
                }),
            Translation(french,
                'Le steak est bon',
                'lesteak',
                {
                    bt_user:('<p>Steak devrait être aussi bon pour vous comme '
                        'it les goûts.</p>'),
                    bt_poll:"""
<h3>Poll: Steak Favourite?</h3>
<ul>
<li>Steak T-Bone</li>
<li>Filet Mignon</li>
<li>Chateaubriand</li>
</ul>
""",
                }),
            ]
        )
        mp.make_default_for_node()
        health.save()
        sub_menu.create_child(mp)

        # create a page without the default translation
        MetaPage.create_translated_page(health, pt_article, [
            Translation(french,
                'Trans-graisses vous tuer',
                'transgraisses',
                {
                    bt_user: ("<p>Trans-gras ve ta tuer, mais ce n'est pas de "
                        'nos jours?</p>'),
                    bt_poll:"""
<h3>Poll: Steak Favourite?</h3>
<ul>
<li>Steak T-Bone</li>
<li>Filet Mignon</li>
<li>Chateaubriand</li>
</ul>
""",
                }),
            ]
        )

        # when creating the smoking pages use the blocks from steak polls
        poll = mp.get_translation(english).blocks.filter(block_type=bt_poll)[0]
        lepoll = mp.get_translation(french).blocks.filter(
            block_type=bt_poll)[0]

        smoking = MetaPage.create_translated_page(health, pt_article, [
            Translation(english,
                'Smoking is bad', 'smoking',
                {
                    bt_user:'<p>Smoking is bad unless you are a salmon.</p>',
                    bt_poll:poll.content,
                }),
            Translation(french,
                'Fumer est mauvais', 'fumer',
                {
                    bt_user:('<p>Fumer est mauvais, sauf si vous êtes un '
                        'saumo.</p>'),
                    bt_poll:lepoll.content,
                })
            ])
        sub_menu.create_child(smoking, {english:'Puffing', french:'Le Puffing'})

        # Smoking alias in Fitness
        smoking.create_alias(fitness)

        # blog pages
        for x in range(1, 40):
            MetaPage.create_page(blog, pt_blog, 'Blog %s' % x, 'blog_%s' % x,
                {
                    bt_blog:'<p>Blog entry %s.</p>' % x,
                }
            )

        # -----------------
        # create a second site
        Site.create_site('Second Site', 'dummyhost', 
            languages=[site.default_language])

        # -----------------
        # create some test users
        UserProfile.create('fflintstone', 'Fred', 'Flintstone', 
            'fred@quarry.com', 'yabbadabbadoo')
        UserProfile.create('brubble', 'Barney', 'Rubble', 
            'barney@quarry.com', 'yabbadabbadoo')

        # create an admin
        profile = UserProfile.create('admin', 'A', 'Admin', 'admin@admin.com', 
            'admin')
        profile.user.is_staff = True
        profile.user.is_superuser = True
        profile.user.save()



# yacon.admin.py

import logging

from django.contrib import admin

from yacon.models.pages import MetaPage
from yacon.models.groupsq import GroupOfGroups

logger = logging.getLogger(__name__)

# =============================================================================
# Admin Classes
# =============================================================================

# =============================================================================
# Add Modules To Admin

admin.site.register(MetaPage)

@admin.register(GroupOfGroups)
class GroupOfGroupsAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')



# groupsq.models.py
#
# GroupSq (Group Squared) works with Django auth to provide groups of groups
# and classes of groups functionality.
import logging

from django.contrib.auth.models import User, Group
from django.db import models

from yacon.models.common import TimeTrackedModel

logger = logging.getLogger(__name__)

# ============================================================================

class CycleDetected(Exception):
    def __init__(self, chain):
        self.chain = chain
        super(CycleDetected, self).__init__()


def _descend_cycle(gog, seen):
    for g in gog.group_of_groups.all():
        if g.id in seen:
            seen.append(g.id)
            raise CycleDetected(seen)

        yield g
        seen.append(g.id)
        for subg in _descend_cycle(g, seen):
            yield subg


def cycle_detect_iterator(gog):
    """Iterator that goes through a GroupOfGroups' nested GroupOfGroups
    field.  If a cycle is an exception is raised.

    :raises: CycleDetected
    """
    seen = [gog.id]
    return _descend_cycle(gog, seen)


class GroupOfGroups_Base(TimeTrackedModel):
    """Base class for a group of groups that contains users, groups or groups 
    of groups."""

    # GofG can contain django users, django groups or another GofG
    users = models.ManyToManyField(User, blank=True)
    groups = models.ManyToManyField(Group, blank=True)
    group_of_groups = models.ManyToManyField(
        'yacon.GroupOfGroups', blank=True)

    class Meta:
        app_label = 'yacon'
        verbose_name = 'MetaPage'
        abstract = True

    # -----------------------------------------
    # Settr Methods
    def add(self, item):
        if isinstance(item, User):
            self.users.add(item)
        elif isinstance(item, Group):
            self.groups.add(item)
        elif isinstance(item, GroupOfGroups):
            self.group_of_groups.add(item)
        else:
            raise AttributeError(
                'parameter to add() was not a User, Group or GroupOfGroups')

        self.save() # force an update of the timestamp

    # -----------------------------------------
    # Gettr Methods

    def _safe_list_users(self):
        """Generator for the non-recursive portion of users in a GofG"""
        for u in self.users.all():
            yield u

        for group in self.groups.all():
            for u in group.user_set.all():
                yield u

    def list_users(self):
        """Generator that returns all users nested within sub-relations"""
        users = set()
        users.update(self._safe_list_users())

        # recurse through any sub relations and find all users
        try:
            for gog in cycle_detect_iterator(self):
                users.update(gog._safe_list_users())
        except CycleDetected as e:
            logger.warning('Cycle detected in %s chain was: %s', 
                self, e.chain)

        return list(users)

    def has_user(self, user):
        """Searches for users recursively within all relations for this GofG
        and returns true if the passed in user is found."""
        for u in self._safe_list_users():
            if u == user:
                return True

        # recurse through any sub relations and find all users
        try:
            for gog in cycle_detect_iterator(self):
                for u in gog._safe_list_users():
                    if u == user:
                        return True
        except CycleDetected as e:
            logger.warning('Cycle detected in %s chain was: %s', 
                self, e.chain)

        return False


class GroupOfGroups(GroupOfGroups_Base):
    """Defines a group of groups that is unique by name"""
    name = models.CharField(max_length=80, unique=True)

    class Meta:
        app_label = 'yacon'
        verbose_name = 'Group Of Groups'
        verbose_name_plural = 'Group Of Groups'

    def __unicode__(self):
        return 'GroupOfGroups(id=%s, %s)' % (self.id, self.name)


class OwnedGroupOfGroups(GroupOfGroups_Base):
    """Defines a group of groups that is owned by a user.  The name parameter
    is no longer needed to be unique.  Used for things like "friends of
    George". """

    owner = models.ForeignKey(User, related_name='ownedgroupofgroups_owner_set')
    name = models.CharField(max_length=80)



# yacon.views.left_control.py
# blame ctrudeau chr(64) arsensa.com
#
# Left pane of Nexus control panel.  Contains Site drop-down and the site
# tree.
#

import logging, json

from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404

from yacon.decorators import superuser_required
from yacon.models.hierarchy import Node, Menu, MenuItem, MenuItemTranslation
from yacon.models.site import Site
from yacon.models.pages import MetaPage

logger = logging.getLogger(__name__)

# ============================================================================
# Tree Building Methods
# ============================================================================

def _pages_subtree(node, language, is_root, depth_limit, expanded):
    """Returns a hash representation in dynatree format of the node passed in
    and its children."""
    name = '%s (%s)' % (node.name, node.slug)
    if node.name == None:
        name = '<i>empty translation (%s)</i>' % language.code

    node_hash = {
        'title': name,
        'key': 'node:%d' % node.id,
        'icon': 'fatcow/folder.png',
    }
    if is_root:
        node_hash['expand'] = True

    if depth_limit == 0 and node_hash['key'] not in expanded:
        # reached as far as we're going to go, just check for kids
        count = MetaPage.objects.filter(node=node).count()
        if count != 0 or node.has_children():
            node_hash['isLazy'] = True

        return node_hash

    # check for pages at this level
    metapages = MetaPage.objects.filter(node=node)
    if len(metapages) != 0:
        children = []
        for metapage in metapages:
            icon = "fatcow/page_white.png"
            if metapage.is_alias():
                icon = "fatcow/page_white_link.png"
            page = metapage.get_default_translation()
            if page == None:
                title = '<i>empty translation (%s)</i>' % language.code
            else:
                title = page.title

            page_hash = {
                'title': title,
                'key': 'metapage:%d' % metapage.id,
                'icon': icon,
            }
            children.append(page_hash)

        if 'children' in node_hash:
            node_hash['children'].extend(children)
        else:
            node_hash['children'] = children

    # process any children of this level
    if node.has_children():
        children = []
        for child in node.get_children():
            dl = depth_limit
            if dl != -1:
                dl = dl - 1
            subtree = _pages_subtree(child, language, False, dl, expanded)
            children.append(subtree)

        if 'children' in node_hash:
            node_hash['children'].extend(children)
        else:
            node_hash['children'] = children

    return node_hash


def _menuitem_subtree(menuitem, language, is_root, depth_limit, expanded):
    """Returns a hash representation in dynatree format of the menuitem passed
    in and its children."""
    try:
        tx = MenuItemTranslation.objects.get(menuitem=menuitem, 
            language=language)
        name = tx.name
    except MenuItemTranslation.DoesNotExist:
        name = '<i>empty translation (%s)</i>' % language.code

    menuitem_node = {
        'title': name,
        'key': 'menuitem:%d' % menuitem.id,
    }
    if is_root:
        menuitem_node['expand'] = True

    if depth_limit == 0 and menuitem_node['key'] not in expanded:
        # reached as far as we're going to go, check for kids
        if menuitem.has_children():
            menuitem_node['isLazy'] = True
            menuitem_node['icon'] = 'fatcow/folder.png'

        return menuitem_node

    if menuitem.has_children():
        children = []
        for child in menuitem.get_children():
            dl = depth_limit
            if depth_limit != -1:
                dl -= 1
            subtree = _menuitem_subtree(child, language, False, dl, expanded)
            children.append(subtree)

        menuitem_node['icon'] = 'fatcow/folder.png'
        menuitem_node['children'] = children

    return menuitem_node


def _build_dynatree(site, expanded):
    """Returns a dynatree hash representation of our pages and menu
    hierarchy."""
    subtree = _pages_subtree(site.doc_root, site.default_language, True, 1,
        expanded)
    subtree['activate'] = True
    pages_node = {
        'title': 'Pages',
        'key': 'system:pages',
        'expand': True,
        'icon': 'fatcow/folders_explorer.png',
        'children': [subtree, ],
    }

    language = site.default_language
    menus = []
    for menu in Menu.objects.filter(site=site):
        items = []
        for item in menu.first_level.all():
            items.append(_menuitem_subtree(item, language, True, 1, expanded))

        menus.append({
            'title': menu.name,
            'key': 'menu:%d' % menu.id,
            'expand': True,
            'icon': 'fatcow/folders.png',
            'children':items,
        })

    menus_node = {
        'title': 'Menus',
        'key': 'system:menus',
        'expand': True,
        'icon': 'fatcow/folders_explorer.png',
        'children': menus,
    }

    tree = [pages_node, menus_node]
    return tree


def _get_expanded(request):
    expanded = []
    if 'expandedKeyList' in request.GET:
        for key in request.GET['expandedKeyList'].split(','):
            key = key.strip()
            if key:
                expanded.append(key)

    return expanded

# ============================================================================
# Control Panel: Site Methods
# ============================================================================

@superuser_required
def get_sites(request):
    sites = Site.objects.all().order_by('id')
    data = {}
    for site in sites:
        data[site.id] = site.name

    return HttpResponse(json.dumps(data), content_type='application/json')

# ============================================================================
# Control Panel: Tree Methods
# ============================================================================

@superuser_required
def tree_top(request, site_id):
    site = get_object_or_404(Site, id=site_id)
    expanded = _get_expanded(request)
    tree = _build_dynatree(site, expanded)

    return HttpResponse(json.dumps(tree), content_type='application/json')


@superuser_required
def tree_top_default_site(request):
    # pick the first site and return it
    sites = Site.objects.all()
    if len(sites) == 0:
        return HttpResponse(json.dumps([]), content_type='application/json')

    return tree_top(request, sites[0].id)


@superuser_required
def sub_tree(request):
    key = request.GET.get('key')
    if not key:
        raise Http404('no key sent')

    (node_type, node_id) = key.split(':')
    if not node_type or not node_id:
        raise Http404('bad key sent: "%s"' % key)

    expanded = _get_expanded(request)
    if node_type == 'node':
        node = get_object_or_404(Node, id=node_id)
        tree = _pages_subtree(node, node.site.default_language, False, 1,
            expanded)
        subtree = tree['children']
    elif node_type == 'menuitem':
        menuitem = get_object_or_404(MenuItem, id=node_id)
        tree = _menuitem_subtree(menuitem, menuitem.menu.site.default_language, 
            False, 1, expanded)
        subtree = tree['children']
    else:
        raise Http404('bad node_type sent: "%s"' % node_type)

    return HttpResponse(json.dumps(subtree), content_type='application/json')



# yacon.decorators.py

import logging
from functools import wraps

from django.contrib.auth.views import redirect_to_login
from django.http import Http404

from yacon.utils import FileSpec, get_profile

logger = logging.getLogger(__name__)

# ============================================================================

def superuser_required(target):
    """Decorator that ensures user is logged in and has superuser bit set."""

    @wraps(target)
    def wrapper(*args, **kwargs):
        request = args[0]

        # test goes here
        if request.user.is_authenticated() and request.user.is_superuser:
            return target(*args, **kwargs)

        # redirect to login page defined in settings with the current URL as
        # the "next" path
        return redirect_to_login(request.build_absolute_uri())
    return wrapper


def post_required(target):
    """Decorator for views that must only be called as POST"""
    @wraps(target)
    def wrapper(*args, **kwargs):
        request = args[0]
        if request.method != 'POST':
            raise Http404('GET method not supported')

        return target(*args, **kwargs)
    return wrapper


def profile_required(target):
    """Ensures request.user has a profile, returns it in the request object.
    Otherwise, raises 404."""
    @wraps(target)
    def wrapper(*args, **kwargs):
        request = args[0]
        profile = get_profile(request.user)
        if profile:
            return target(*args, **kwargs)

        raise Http404('no profile for user')
    return wrapper


def verify_node(parm, is_file):
    """Decorator to check user's permissions against a file node contained in
    the query string.  Does a request.GET[parm] to check the node in the view.
    A FileSpec is created based on the node, if the permission check passes
    then this spec is put into the request (arg0).  

    If the user is a superuser then the permissions are granted.  If not, the
    node is checked against the user in the request.  See the
    FileSpec.allowed_for_user() method for rules on whether a user can see a
    file node.
    
    The "is_file" parameter is a boolean, True indicates the node is a file.
    """
    def decorator(target):
        @wraps(target)
        def wrapper(*args, **kwargs):
            # process options
            request = args[0]
            node = request.GET.get(parm)
            if not node:
                raise Http404('permission denied')

            spec = FileSpec(node, node_is_file=is_file)

            if spec.allowed_for_user(request.user):
                request.spec = spec
                return target(*args, **kwargs)

            logger.error('user %s attempted to access node %s',
                request.user.username, node)

            raise Http404('permission denied')

        return wrapper
    return decorator


def verify_file_url(parm, is_file):
    """Decorator to check user's permissions against a file url contained in 
    the query string.  Does a request.GET[parm] to check the file in the view.
    A FileSpec is created based on the file, if the permission check passes
    then this spec is put into the request (arg0).  

    If the user is a superuser then the permissions are granted.  If not, the
    file is checked against the user in the request.  See
    FileSpec.allowed_for_user() for the rules on whether is granted permission.

    The "is_file" parameter is a boolean, True indicates the url is for a file.
    """
    def decorator(target):
        @wraps(target)
        def wrapper(*args, **kwargs):
            # process options
            request = args[0]
            url = request.GET.get(parm)
            if not url:
                raise Http404('permission denied')

            spec = FileSpec.factory_from_url(url, ensure_file=is_file)

            if spec.allowed_for_user(request.user):
                request.spec = spec
                return target(*args, **kwargs)

            logger.error('user %s attempted to access %s',
                request.user.username, url)

            raise Http404('permission denied')

        return wrapper
    return decorator



from yacon.models.common import TimeTrackedModel, Language
from yacon.models.content import (PermissionHandler, AlwaysYesPermissionHandler,
    ContentRenderingException, ContentHandler, FlatContent, DynamicContent)
from yacon.models.groupsq import GroupOfGroups, OwnedGroupOfGroups
from yacon.models.hierarchy import (BadSlug, Node, NodeTranslation, MenuItem,
    Menu, MenuItemTranslation)
from yacon.models.pages import (PageType, BadContentHandler, BlockType, Block,
    Translation, Page, DoubleAliasException, MetaPage)
from yacon.models.site import ParsedPath, Site, SiteConfig
from yacon.models.users import UsernameError, UserProfile

pyflakes = (TimeTrackedModel, Language, PermissionHandler, 
    AlwaysYesPermissionHandler, ContentRenderingException, ContentHandler, 
    FlatContent, DynamicContent, GroupOfGroups, OwnedGroupOfGroups, BadSlug, 
    Node, NodeTranslation, MenuItem, Menu, MenuItemTranslation, PageType, 
    BadContentHandler, BlockType, Block, Translation, Page, 
    DoubleAliasException, MetaPage, ParsedPath, Site, SiteConfig, 
    UsernameError, UserProfile)



from django.core.management.base import BaseCommand

from yacon.models.common import Language
from yacon.models.site import Site

class Command(BaseCommand):
    def handle(self, *args, **options):
        name='Localhost Site'
        domain='localhost:8000'

        language = Language.factory(name='English', identifier='en')
        Site.create_site(name, domain, languages=[language])



# yacon.forms.py
#
import logging

from django import forms

from yacon.definitions import TITLE_LENGTH, SLUG_LENGTH
from yacon.utils import get_user_attributes

logger = logging.getLogger(__name__)

# ============================================================================
# Forms
# ============================================================================

class UpdateUserForm(forms.Form):
    username = forms.RegexField(max_length=30, regex=r'^[\w.@+-]+$',
        error_messages = {'invalid':('Only letters, numbers and the characters '
            '@ . + - _ are allowed.')})
    first_name = forms.CharField(max_length=30)
    last_name = forms.CharField(max_length=30)
    email = forms.EmailField(max_length=75)
    is_active = forms.BooleanField(required=False, initial=True)
    is_staff = forms.BooleanField(required=False)
    is_superuser = forms.BooleanField(required=False)

    def __init__(self, *args, **kwargs):
        if 'initial' not in kwargs:
            initial =  {}

        if 'user' in kwargs:
            user = kwargs.pop('user')
            initial.update({
                'username':user.username,
                'first_name':user.first_name,
                'last_name':user.last_name,
                'email':user.email,
                'is_active':user.is_active,
                'is_staff':user.is_staff,
                'is_superuser':user.is_superuser,
            })

        if 'profile' in kwargs:
            profile = kwargs.pop('profile')

            for attr in get_user_attributes(profile):
                if attr == 'user':
                    continue

                initial[attr] = getattr(profile, attr)

        kwargs['initial'] = initial
        super(UpdateUserForm, self).__init__(*args, **kwargs)


class AddUserForm(UpdateUserForm):
    password1 = forms.CharField(max_length=30, widget=forms.PasswordInput())
    password2 = forms.CharField(max_length=30, widget=forms.PasswordInput())

    def clean(self):
        cleaned_data = super(AddUserForm, self).clean()
        if 'password1' not in cleaned_data or 'password2' not in cleaned_data:
            raise forms.ValidationError('Password cannot be empty')

        if cleaned_data['password1'] != cleaned_data['password2']:
            raise forms.ValidationError('Password mismatch')

        return cleaned_data


class CreatePageForm(forms.Form):
    title = forms.CharField(max_length=TITLE_LENGTH)
    slug = forms.CharField(max_length=SLUG_LENGTH, required=False)
    auto_slug = forms.BooleanField(required=False)

    EXPECTED_FIELDS = ('title', 'slug', 'auto_slug', 'csrfmiddlewaretoken')

    def __init__(self, *args, **kwargs):
        super(CreatePageForm, self).__init__(*args, **kwargs)

        self.fields['auto_slug'].widget = forms.widgets.HiddenInput()
        auto_slug = self.initial.get('auto_slug', False)
        if auto_slug:
            self.fields['slug'].widget = forms.widgets.HiddenInput()

        # populate the extra fields passed in, args0=POST
        if args and args[0]:
            for key, value in args[0].items():
                if key in self.EXPECTED_FIELDS:
                    continue

                self.fields[key] = forms.CharField(required=False)


    def clean(self):
        data = super(CreatePageForm, self).clean()
        if not data['auto_slug'] and not data['slug']:
            # auto slug not specified but slug not given
            raise forms.ValidationError('slug field cannot be empty')

        return data



from django.core.management.base import BaseCommand, CommandError

from yacon.models.site import Site

class Command(BaseCommand):
    def handle(self, *args, **options):
        if len(args) != 2:
            raise CommandError(('Must provide two arguments: "name" and '
                '"domain"'))

        Site.create_site(name=args[0], domain=args[1])



# yacon.context_processors.py
import logging

from django.conf import settings
from yacon import conf
from yacon.models.site import Site

logger = logging.getLogger(__name__)

# =============================================================================

def processor(request):
    data = {
        'admin_enabled': 'django.contrib.admin' in settings.INSTALLED_APPS,
        'conf':conf,
        'request':request,
        'site':Site.get_site(request),
    }

    if hasattr(request, 'user'):
        data['user'] = request.user
        if request.user.is_authenticated():
            data['authenticated'] = True
            if request.user.is_superuser:
                data['superuser'] = True

    return data



import os
from PIL import Image

from django.conf import settings
from django.core.management.base import BaseCommand

from yacon import conf

class Command(BaseCommand):
    def handle(self, *args, **options):
        if not conf.site.auto_thumbnails:
            print 'Auto thumbnails not configured, exiting'
            quit()

        thumb_dir = conf.site.auto_thumbnails['dir']
        thumb_conf = conf.site.auto_thumbnails['config']

        for dirpath, dirnames, filenames in os.walk(settings.MEDIA_ROOT):
            print 'processing dirpath: %s' % dirpath
            if thumb_dir in dirpath:
                print '   skipping, thumb_dir in dirpath'
                continue

            for filename in filenames:
                print '   processing filename:', filename
                extension = None
                if '.' in filename:
                    pieces = filename.split('.')
                    extension = pieces[-1]

                if extension not in conf.site.image_extensions:
                    print '      skipping, non image extension'
                    continue

                # if you get here then the file has an image extension
                full_filename = os.path.join(dirpath, filename)
                for thumb_type, size in thumb_conf.items():
                    image_dir = os.path.realpath(os.path.join(dirpath, 
                        thumb_dir, thumb_type))
                    image_name = os.path.join(image_dir, filename)
                    if os.path.exists(image_name):
                        print '      thumbnail already exists'
                        continue
                    try:
                        os.makedirs(image_dir)
                    except:
                        pass # already exists, do nothing

                    # use PIL to create the thumbnail
                    im = Image.open(full_filename)
                    im.thumbnail(size, Image.ANTIALIAS)
                    im.save(image_name, 'png')
                    print '      created thumbnail:', image_name



# yacon.views.browser.py
#
# File Browser Views

import os, logging, json, shutil, operator, urllib
from io import FileIO, BufferedWriter
from PIL import Image

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt

from yacon import conf
from yacon.decorators import verify_node, verify_file_url
from yacon.utils import FileSpec, files_subtree, build_filetree, JSONResponse

logger = logging.getLogger(__name__)

# ============================================================================

BUFFER_SIZE = 10485760  # 10MB

# ============================================================================
# Upload Helpers
# ============================================================================

def _upload_save(request, spec):
    """Handles the uploading of a file from the Valum Uploader widget."""
    try:
        if request.is_ajax():
            # using AJAX upload, get the filename out of the query string
            filename = request.GET['qqfile']
            # check that the filename doesn't do anything evil
            if os.path.normpath(filename) != filename:
                logger.error('user sent bad filename: %s', filename)
                raise Http404('bad filename')

            spec.set_filename(filename)

            try:
                os.makedirs(spec.full_dir)
            except OSError:
                pass

            dest = BufferedWriter(FileIO(spec.full_filename, "w"))
            chunk = request.read(BUFFER_SIZE)
            while len(chunk) > 0:
                dest.write(chunk)
                chunk = request.read(BUFFER_SIZE)
        else:
            # not an AJAX request, using iframe method which means there will
            # only be one file at a time, use the first one

            uploaded = request.FILES.values()[0]
            filename = uploaded.name
            spec.set_filename(filename)

            # ??? HAVE TO DO SOMETHING ABOUT THE FILENAME
            # don't forget the normpath check

            dest = BufferedWriter(FileIO(spec.full_filename, "w"))
            for chunk in uploaded.chunks():
                dest.write(chunk)
    except:
        # normally wouldn't use 404 for this kind of error but it allows us to
        # treat it as an exception instead of handling different returns, the
        # widget doesn't care, if it doesn't get back json({success}) is
        # reports an error
        logger.exception('Upload caused exception')
        raise Http404('Upload caused exception')

    return spec


def _handle_upload(request, prefix=None):
    if request.method == 'GET':
        raise Http404('GET not supported for upload_file')

    node = request.GET.get('node')
    if not node:
        raise Http404('No node sent with upload')

    spec = FileSpec(node, prefix=prefix)
    if not spec.allowed_for_user(request.user):
        raise Http404('permission denied')

    _upload_save(request, spec)

    # check if upload had an image extension
    if not spec.extension:
        return spec

    if spec.extension.lower() not in conf.site.image_extensions:
        return spec

    # create thumbnails for the image if configured
    if len(conf.site.auto_thumbnails) == 0:
        return spec

    try:
        for key, value in conf.site.auto_thumbnails['config'].items():
            spec.make_thumbnail(conf.site.auto_thumbnails['dir'], 
                key, value[0], value[1])

    except KeyError, e:
        logger.error(('auto_thumbnails missing conf key stopping thumbnail '
            'generation'), e.message)

    return spec

# ============================================================================
# Browser Views
# ============================================================================

@login_required
def ckeditor_browser(request):
    func_num = request.GET.get('CKEditorFuncNum', None)
    if not func_num:
        raise Http404('CKEditorFuncNum must be provided')

    image_only = request.GET.get('image_only', False)
    if image_only == "1":
        image_only = True

    request.session['func_num'] = func_num
    request.session['choose_mode'] = 'ckeditor'
    request.session['image_only'] = image_only
    request.session['popup'] = True
    data = {
        'title':'Uploads',
        'base_template':'yacon/browser_base.html',
        'popup':True,
    }
    return render_to_response('yacon/browser/browser.html', data, 
        context_instance=RequestContext(request))


@login_required
def popup_browser(request, callback):
    image_only = request.GET.get('image_only', False)
    if image_only == "1":
        image_only = True

    choose_mode = request.GET.get('choose_mode', 'singleselect')

    request.session['callback'] = callback
    request.session['choose_mode'] = choose_mode
    request.session['image_only'] = image_only
    request.session['popup'] = True
    data = {
        'title':'File Browser',
        'base_template':'yacon/browser_base.html',
        'popup':True,
    }
    return render_to_response('yacon/browser/browser.html', data, 
        context_instance=RequestContext(request))


# ============================================================================
# Browser Ajax Views
# ============================================================================

@login_required
def root_control(request, tree_type):
    data = {
        'tree_type':tree_type,
        'is_superuser':request.user.is_superuser,
    }
    return render_to_response('yacon/browser/root_control.html', data, 
        context_instance=RequestContext(request))


@login_required
@csrf_exempt
def upload_file(request):
    spec = _handle_upload(request)
    return HttpResponse(spec.json_results)


@login_required
@csrf_exempt
def user_upload_file(request):
    prefix = 'users/%s' % request.user.id
    spec = _handle_upload(request, prefix=prefix)
    return HttpResponse(spec.json_results)


@login_required
def tree_top(request):
    expanded = []
    if 'expandedKeyList' in request.GET:
        for key in request.GET['expandedKeyList'].split(','):
            key = key.strip()
            if key:
                expanded.append(key)

    restricted = None
    if not request.user.is_superuser:
        restricted = request.user.id

    try:
        tree = build_filetree(expanded, restricted)
    except OSError:
        tree = ['File error, is MEDIA_ROOT set?']

    return HttpResponse(json.dumps(tree), content_type='application/json')


@login_required
def sub_tree(request):
    key = request.GET.get('key')
    if not key:
        raise Http404('no key sent')

    spec = FileSpec(key)
    tree = files_subtree(spec, 1, [])
    if 'children' in tree:
        subtree = tree['children']
    else:
        subtree = []

    return HttpResponse(json.dumps(subtree), content_type='application/json')


class StubFile(object):
    pass


@login_required
@verify_node('node', False)
def show_folder(request):
    spec = request.spec    # verify_node puts this in the request
    image_only = request.session.get('image_only', False)
    popup = request.session.get('popup', False)
    base_url = settings.MEDIA_URL
    if not base_url:
        logger.warning('MEDIA_URL is not set.  Uploads will not show properly')

    if spec.file_type == 'private':
        base_url = conf.site.private_upload_url

    relative_url = base_url + spec.relative_dir + '/'

    files = []
    images = []
    for x in os.listdir(spec.full_dir):
        if os.path.isdir(os.path.join(spec.full_dir, x)):
            continue

        stub = FileSpec.factory_from_path('%s/%s' % (spec.full_dir, x))
        stub.lower_name = stub.basename.lower()

        if stub.is_image:
            images.append(stub)
        else:
            if not image_only:
                files.append(stub)

    images.sort(key=operator.attrgetter('lower_name'))
    files.sort(key=operator.attrgetter('lower_name'))

    data = {
        'title':'Folder Info',
        'node':request.GET['node'],
        'spec':spec,
        'relative_url':relative_url,
        'files':files,
        'images':images,
        'image_extensions':json.dumps(conf.site.image_extensions),
        'func_num':request.session.get('func_num', None),
        'choose_mode':request.session.get('choose_mode', 'view'),
        'callback':request.session.get('callback', ''),
        'image_only':image_only,
        'popup':popup,
    }

    return render_to_response('yacon/browser/show_folder.html', data, 
        context_instance=RequestContext(request))


@login_required
@verify_node('node', False)
def add_folder(request, name):
    spec = request.spec    # verify_node puts this in the request
    dir_path = os.path.join(spec.full_dir, name)
    os.mkdir(dir_path)

    return HttpResponse()


@login_required
@verify_node('node', False)
def remove_folder_warn(request):
    """Ajax call that returns a listing of the directories and files that
    would be effected if the given folder was removed."""
    spec = request.spec    # verify_node puts this in the request

    files = []
    dirs = []
    for dirpath, dirnames, filenames in os.walk(spec.full_dir):
        for filename in filenames:
            stored_name = os.path.join(spec.relative_dir, filename)
            files.append(stored_name)

        for dirname in dirnames:
            stored_name = os.path.join(spec.relative_dir, dirname)
            dirs.append(stored_name)

    data = {
        'files':files,
        'dirs':dirs,
        'spec':spec,
        'node':request.GET['node'],
    }

    return render_to_response('yacon/browser/remove_folder_warning.html', 
        data, context_instance=RequestContext(request))


@login_required
@verify_node('node', False)
def remove_folder(request):
    spec = request.spec    # verify_node puts this in the request
    shutil.rmtree(spec.full_dir)
    return HttpResponse()


@login_required
@verify_node('node', True)
def remove_file(request):
    spec = request.spec    # verify_node puts this in the request
    if not os.path.exists(spec.full_filename):
        logger.error('ignored request to remove non-existent %s',
            spec.full_filename)
        return HttpResponse()

    os.remove(spec.full_filename)

    # search for any matching files with the same name in any thumbnails
    # directories
    if len(conf.site.auto_thumbnails) == 0:
        return HttpResponse()

    try:
        thumb_path = os.path.abspath(os.path.join(spec.full_dir, 
            conf.site.auto_thumbnails['dir']))
        for path, dirs, files in os.walk(thumb_path):
            for filename in files:
                if os.path.basename(filename) == spec.basename:
                    full_filename = os.path.abspath(os.path.join(path, 
                        filename))
                    os.remove(full_filename)

    except KeyError, e:
        logger.error(('auto_thumbnails missing conf key stopping thumbnail '
            'removal'), e.message)

    return HttpResponse()


@login_required
@verify_file_url('image', True)
def image_edit(request):
    data = {
        'spec':request.spec, # verify_file_url puts this in the request
    }
    if not os.path.exists(request.spec.full_filename):
        return render_to_response('yacon/browser/image_edit_error.html', data, 
            context_instance=RequestContext(request))

    return render_to_response('yacon/browser/image_edit.html', data, 
        context_instance=RequestContext(request))


@login_required
@verify_file_url('image', True)
def image_edit_save(request):
    spec = request.spec
    if not os.path.exists(request.spec.full_filename):
        return HttpResponse('File not found on server', status='404')

    # use PIL to rotate, scale and crop the image
    changed = False
    img = Image.open(spec.full_filename)
    rotate = request.GET.get('rotate')
    if rotate:
        img = img.rotate(-1 * int(rotate))
        changed = True

    width = request.GET.get('scale_width')
    height = request.GET.get('scale_height')
    if width:
        img = img.resize((int(float(width)), int(float(height))), 
            Image.ANTIALIAS)
        changed = True

    x1 = request.GET.get('crop_x1')
    y1 = request.GET.get('crop_y1')
    x2 = request.GET.get('crop_x2')
    y2 = request.GET.get('crop_y2')
    if x1:
        box = (int(float(x1)), int(float(y1)), int(float(x2)), int(float(y2)))
        img = img.crop(box)
        changed = True
        
    if not changed:
        return HttpResponse()

    # image changed, saved the new one and update any thumbnails
    img.save(spec.full_filename)
    try:
        for key, value in conf.site.auto_thumbnails['config'].items():
            spec.make_thumbnail(conf.site.auto_thumbnails['dir'], 
                key, value[0], value[1])

    except KeyError, e:
        logger.error(('auto_thumbnails missing conf key stopping thumbnail '
            'generation'), e.message)

    return HttpResponse()


@login_required
@verify_file_url('file', True)
def file_expand(request):
    data = {
        'spec':request.spec, # verify_file_url puts this in the request
    }
    if not os.path.exists(request.spec.full_filename):
        result = {
            'success':False,
            'msg':'Bad filename',
        }
    else:
        try:
            request.spec.expand_file()
            result = {
                'success':True,
            }
        except Exception as e:
            result = {
                'success':False,
                'msg':e.message,
            }

    return JSONResponse(result, extra_headers={'Cache-Control':'no-cache'})



from setuptools import setup, find_packages

setup(
    name='django-yacon',
    version='0.6.1',
    description='Django based Content Managment building framework',
    long_description=('Yacon is a django app for building websites based on '
        'a CMS.  Yacon provides methods for managing hierarchical, '
        'multi-lingual page content across multple sites.  Yacon ships with '
        'ckedit and uses AJAX calls to allow the user to do inline WYSIWYG '
        'editing of their web content.  Yacon does not provide any design '
        'or templating tools, and so is not a full CMS, but provides '
        'enough to support CMS features on any django site you are already '
        'building.'),
    url='https://github.com/cltrudeau/django-yacon',
    author='Christopher Trudeau',
    author_email='ctrudeau+pypi@arsensa.com',
    license='MIT',
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='django,CMS',
    packages=find_packages(),
    install_requires=[
        'Django>=1.7',
        'Pillow>=2.7',
        'bleach>=1.4',
        'django-treebeard>=3',
    ],
)



# yacon.yacon_tags.py
# blame ctrudeau chr(64) arsensa.com
#
# Defines specialized template tags for the rendering of content from the CMS

import logging, traceback, json

from django import template
from django.template import loader

from yacon import conf
from yacon.models.pages import BlockType
from yacon.models.hierarchy import Menu
from yacon.utils import SummarizedPage, get_system_text

register = template.Library()
logger = logging.getLogger(__name__)

# ============================================================================
# Utility Methods
# ============================================================================

# templates for blocks
templates = {
    'editable': loader.get_template('yacon/blocks/editable.html'),
    'title_editable': loader.get_template('yacon/blocks/title_editable.html'),
    'createable': loader.get_template('yacon/blocks/createable.html'),
    'page_last_updated': loader.get_template(
        'yacon/blocks/page_last_updated.html'),

    # errors
    'no_such_block_type': 
        loader.get_template('yacon/blocks/errors/no_such_block_type.html'),
    'no_such_block': loader.get_template(
        'yacon/blocks/errors/no_such_block.html'),
    'exception': loader.get_template('yacon/blocks/errors/exception.html'),
}


def _render_block_by_key(context, tag_name, key):
    """Searches for a block within a given page.  Returns a tuple of (boolean
    success, rendered tag).  Expects a tag parameter called "key" and a
    template variable called "page", which is a Page object.

    :param tag_name: name of tag being rendered
    :param key: key of block to be rendered

    :returns: (boolean, string) tuple indicating success and resulting text.
        Success may be False and still have content which would be the error
        to present to the user.
    """
    # fetch the blocks for this page
    logger.debug('key is: %s' % key)
    blocks = context['page'].blocks.filter(block_type__key=key)

    context['tag_name'] = tag_name
    context['tag_parameters'] = {
        'key':key,
    }

    if len(blocks) == 0:
        # block not found
        return (False, templates['no_such_block'].render(context))

    # grab the first block sent in
    block = blocks[0]
    try:
        context['block'] = block
        request = context['request']
        return (True, block.render(request, context))
    except Exception:
        context['exception'] = traceback.format_exc()
        return (False, templates['exception'].render(context))


def _render_menuitem(menuitem, language, selected, last, separator, indent):
    spacing = indent * '    '
    results = []
    translations = menuitem.menuitemtranslation_set.filter(language=language)

    li_class = ''
    if menuitem.metapage:
        page = menuitem.metapage.get_translation(language=language)
        if page:
            name = page.title
            if len(translations) != 0:
                name = translations[0].name

            content = '<a href="%s">%s</a>' % (page.uri, name)
        else:
            content = '<i>empty translation (%s)</i>' % language.code
            if len(translations) != 0:
                content = translations[0].name
    elif menuitem.link:
        if len(translations) == 0:
            text = '<i>empty translation (%s)</i>' % language.code
        else:
            text = translations[0].name

        content = '<a href="%s">%s</a>' % (menuitem.link, text)
    else:
        if len(translations) == 0:
            content = '<i>empty translation (%s)</i>' % language.code
        else:
            content = translations[0].name

    if menuitem.metapage == selected:
        li_class = ' class="selected"'

    if last:
        separator = ''

    li = '%s<li%s> %s%s</li>' % (spacing, li_class, content, separator)
    results.append(li)

    children = menuitem.get_children()
    if len(children) != 0:
        results.append('%s<ul>' % spacing)
        for child in menuitem.get_children():
            subitems = _render_menuitem(child, language, selected, last,
                separator, indent + 1)
            results.append(subitems)

        results.append('%s</ul>' % spacing)

    return '\n'.join(results)


def _valum_widget(url, node, extensions=None, on_complete=''):
    allowed_ext = ''
    if extensions:
        allowed_ext = 'allowedExtensions: %s,' % extensions

    if on_complete:
        on_complete = 'onComplete: %s,' % on_complete

    return \
"""
    <script src="/static/yacon/js/valum_uploader/fileuploader.js" ></script>

    <script type="text/JavaScript">
        $(document).ready(function(){
            var uploader = new qq.FileUploader({
                action: "%s",
                element: $('#file-uploader')[0],
                multiple: true,
                %s
                %s
                params: {
                    'node':'%s'
                }
              }) ;
        });
    </script>

    <link href="/static/yacon/js/valum_uploader/fileuploader.css" 
      media="screen" rel="stylesheet" type="text/css"/>

    <div id="file-uploader">       
    </div>
""" % (url, allowed_ext, on_complete, node)

# ============================================================================
# Template Tags
# ============================================================================

@register.simple_tag(takes_context=True)
def block_by_key(context, key):
    """Searches for and renders a block whose associated BlockType object has
    a key with the given name.  The search is done on a Page object passed in
    via the context (where the page is named 'page').  If more than one block
    has the same key then an indetereminate single block is still rendered.

    Both a hardcoded string or a template variable can be used to specify the
    key.  If no block is found or there is an error rendering the block an
    error message is rendered instead.  

    Block rendering is done through the Block's BlockType's ContentHandler
    object.  The rendering method requires the request, uri, node and slugs of
    which the request, uri and slugs have to be in the context.

    Example:
    {% block_by_key "poll" %}

    The above renders the first block with the key "poll" for the associated
    page in the context.

    Example:
    {% block_by_key key %}

    The above renders the first block whose key name matches the contents of
    the template variable 'key'.  If the variable contained "poll" it would be
    equivalent to the previous example.
    """
    (success, content) = _render_block_by_key(context, 'block_by_key', key)
    return content


@register.simple_tag(takes_context=True)
def editable_block_by_key(context, key):
    """If the tag is rendering in a page being created then this renders the
    appropriate wrappers for content for the given block key.  If it is in
    display/edit more then this function performs same actions as
    :func:`block_by_key` except the resulting content is wrapped in order to
    be used by the ajax editing functions.

    The template for wrapping blocks in create mode is found in
    "blocks/createable.html", for display and edit the content is found in
    "blocks/editable.html".  Wrapper content is loaded using the django
    template loader so it can be overloaded.
    """
    page = context['page']
    code = ''
    if page:
        code = page.language.code

    context['button_text'] = {}
    tags = ['content_edit_button', 'content_save_button',
        'content_cancel_button']
    for tag in tags:
        context['button_text'][tag] = get_system_text(code, tag)

    create_mode = context.get('create_mode', False)
    if create_mode:
        page_type = context['page_type']
        block_type = None
        for bt in page_type.block_types.all():
            if bt.key == key:
                block_type = bt
                break

        if not block_type:
            return templates['no_such_block_type'].render(context)

        context['key'] = key
        context['block_type_id'] = block_type.id
        content = templates['createable'].render(context)
    else:
        (success, content) = _render_block_by_key(context, 'block_by_key', key)
        if success:
            # didn't get an error, wrap the content using the editable template 
            context['content'] = content
            content = templates['editable'].render(context)

    return content


@register.simple_tag(takes_context=True)
def editable_page_title(context, page):
    """Prints the title of the page for the given page, wrapping the 
    title in a widget that allows editing via ajax.  
    
    The template for wrapping the content is found in 
    "blocks/title_editable.html" and is loaded using the django template 
    loader so it can be overloaded.
    """
    page = context['page']
    page = context['page']
    code = ''
    if page:
        code = page.language.code

    context['button_text'] = {}
    tags = ['title_edit_button', 'title_save_button', 'title_cancel_button',
        'show_page', 'hide_page']
    for tag in tags:
        context['button_text'][tag] = get_system_text(code, tag)
    create_mode = context.get('create_mode', False)
    if create_mode:
        # don't show this tag when in create mode, handled by create form
        return ''

    context['page'] = page
    content = templates['title_editable'].render(context)
    return content


@register.simple_tag(takes_context=True)
def page_last_updated(context, page):
    """Returns the most recent timestamp of the given page or all of its
    blocks.  Content is wrapped in a template with divs that yacon/editable.js
    is aware of to change the value via ajax when something on the page is
    edited.
    
    The template for wrapping the content is found in 
    "blocks/page_last_updated.html" and is loaded using the django template 
    loader so it can be overloaded.
    """
    context['page'] = page
    content = templates['page_last_updated'].render(context)
    return content


@register.simple_tag(takes_context=True)
def menu(context, name, separator=''):
    """Returns the <li> and nested <ul> representation of the named menu.
    Note that this does not include the surround <ul> tags, this is on purpose
    so that you can add content in the templates."""
    results = []

    try:
        menu = Menu.objects.get(site=context['site'], name=name)
    except Menu.DoesNotExist:
        logger.error('no menu matching name "%s"', name)
        return ''

    page = context.get('page', None)
    if page:
        language = page.language
        select = page.metapage
    else:
        language = context['site'].default_language
        select = None

    user = context.get('user', None)
    items = list(menu.first_level.all())
    for item in items:
        if item.requires_login and (not user or not user.is_authenticated()):
            # login required to see this item, there is no user in the session
            # or they're not authenticated, skip the item
            continue

        if item.requires_admin and (not user or not user.is_superuser):
            continue

        last = (item == items[-1])
        menuitem = _render_menuitem(item, language, select, last, separator, 1)
        results.append(menuitem)

    return '\n'.join(results)


@register.simple_tag(takes_context=True)
def dynamic_content_by_key(context, key):
    """DynamicContent content handlers do not require any Block to be stored
    in the database, but can be fired directly.  A BlockType is registered
    as usual, but this method, unlike block_by_key, doesn't look for a Block,
    but renders the ContentHandler directly."""
    # fetch the named block type
    logger.debug('key is: %s' % key)
    context['tag_name'] = 'dynamic_content_by_key'
    context['tag_parameters'] = {
        'key':key,
    }
    try:
        block_type = BlockType.objects.get(key=key)
        request = context['request']
        handler = block_type.get_content_handler()
        return handler.render(request, context, None)
    except BlockType.DoesNotExist:
        return templates['no_such_block_type'].render(context)
    except Exception:
        context['exception'] = traceback.format_exc()
        return templates['exception'].render(context)


@register.simple_tag(takes_context=True)
def tsort(context, name):
    """Used to produce the ascending/descending links for sorting a table.

    :param name: name of the tag to print the tsort for, if the request object
        contains a "sort" parameter that matches this name then the links will
        show selection appropriately

    :returns: html containing links and images for sorting a table
    """
    request = context['request']
    sort = request.GET.get('sort')
    if not sort:
        sort = context.get('default_sort')
    direction = request.GET.get('direction')

    request.path

    lines = [
        '<a href="%s?sort=%s">' % (request.path, name), 
        '<img ',
    ]
    if sort == name and direction != 'rev':
        lines.append('class="selected" ')

    lines.extend([
        'src="/static/yacon/icons/fatcow/sort_down.png">',
        '</a>',
        '<a href="%s?sort=%s&direction=rev">' % (request.path, name),
        '<img '
    ])

    if sort == name and direction == 'rev':
        lines.append('class="selected" ')

    lines.extend([
        'src="/static/yacon/icons/fatcow/sort_up.png">',
        '</a>',
    ])

    return ''.join(lines)


@register.simple_tag()
def upload_widget(node, on_complete=''):
    """Returns a Valum Uploader widget wired to the general purpose uploading
    URL.

    :param node: storage type (public or private) and path indicator, e.g.
        "public:foo/bar" to have the uploaded file go in MEDIA_ROOT/foo/bar.
    :param on_complete: name of Javascript function to call when an upload has
        complete, will be called with signature:
        function(String id, String fileName, Object responseJSON)
    """
    return _valum_widget('/yacon/browser/upload_file/', node,
        on_complete=on_complete)


@register.simple_tag()
def user_upload_widget(node, on_complete=''):
    """Returns a Valum Uploader widget that uploads files based on the user's
    home directory.

    :param node: storage type (public or private) and path indicator, e.g.
        "public:foo/bar" to have the uploaded file go in 
        MEDIA_ROOT/$USERNAME/foo/bar.
    :param on_complete: name of Javascript function to call when an upload has
        complete, will be called with signature:
        function(String id, String fileName, Object responseJSON)
    """
    return _valum_widget('/yacon/browser/user_upload_file/', node,
        on_complete=on_complete)


@register.simple_tag()
def image_upload_widget(node, on_complete=''):
    """Returns a Valum Uploader widget that uploads images.

    :param node: storage type (public or private) and path indicator, e.g.
        "public:foo/bar" to have the uploaded file go in MEDIA_ROOT/foo/bar.
    :param on_complete: name of Javascript function to call when an upload has
        complete, will be called with signature:
        function(String id, String fileName, Object responseJSON)
    """
    extensions = json.dumps(conf.site.image_extensions)

    return _valum_widget('/yacon/browser/upload_file/', node, 
        extensions=extensions, on_complete=on_complete)


@register.simple_tag()
def user_image_upload_widget(node, on_complete=''):
    """Returns a Valum Uploader widget that uploads images based on the user's
    home directory.

    :param node: storage type (public or private) and path indicator, e.g.
        "public:foo/bar" to have the uploaded file go in MEDIA_ROOT/foo/bar.
    :param on_complete: name of Javascript function to call when an upload has
        complete, will be called with signature:
        function(String id, String fileName, Object responseJSON)
    """
    extensions = json.dumps(conf.site.image_extensions)

    return _valum_widget('/yacon/browser/user_upload_file/', node, 
        extensions=extensions, on_complete=on_complete)


@register.simple_tag(takes_context=True)
def regroup(context, listing, size):
    """Takes a list and creates a new list of lists that are "size" long and
    puts it in a variable named "grouped".  This is done in a tag so that it
    can be done after pagination for performance reasons."""

    grouped = []
    subgroup = []
    count = 0
    for item in listing:
        if count == size:
            grouped.append(subgroup)
            subgroup = []
            count = 0

        subgroup.append(item)
        count += 1

    if subgroup:
        grouped.append(subgroup)

    context['grouped'] = grouped
    return ''


@register.simple_tag(takes_context=True)
def summarize_pages(context, pages, block_key, summary_size):
    """Takes a list of Page objects and creates a list of SummarizePage
    objects.  This is done in a tag as it should be done after pagination for
    performance reasons.  Puts a variable in the context called "summaries".

    :param pages: list of pages to convert
    :param block_key: name of block key used to produce summary text
    :param summary_size: number of characters to include in the summary block

    :returns: nothing, inserts a "summaries" list in the context
    """
    summaries = []
    for page in pages:
        summaries.append(SummarizedPage(page, block_key, summary_size))

    context['summaries'] = summaries

    return ''



# yacon.models.site.py
import re, logging

from django.db import models
from django.shortcuts import get_object_or_404

from yacon.models.common import Language, TimeTrackedModel
from yacon.models.pages import Page
from yacon.models.hierarchy import Node, NodeTranslation

logger = logging.getLogger(__name__)

MATCH_SLASH = re.compile('/')

# ============================================================================
# Utility Classes
# ============================================================================

class ParsedPath(object):
    """Temporary object representing a URI path that has been parsed into a
    series of slugs.  

    :param path: path that was parsed
    :param slugs_in_path: a list of slugs in the path passed in
    :param slugs_after_item: a list of slugs found in the path
        after we ran out of tree to walk
    :param language: Language object for the slug where final item
        object is found.  In theory the Language should be the same 
        for all slugs, but in practice this isn't enforced
    :param node: if the end of the path walked is a Node then this
        will contain that Node object
    :param page: if the end of the path walked is a translated 
        page or a Node with a default_mteapage setting then this will contain 
        the corresponding Page object
    :param item_type: one of:
        * UNKNOWN: if walking the path did not result in a page or Node
        * NODE: if walking the path resulted in a Node
        * PAGETRANSLATION: if the walking the path resulted in a page
            or a Node with a default_page setting
    """
    UNKNOWN = 0
    NODE = 1
    PAGE = 2
    ITEM_TYPES = {
        UNKNOWN:'Unknown',
        NODE:'Node',
        PAGE:'Page',
    }

    def __init__(self, *args, **kwargs):
        self.slugs_in_path = []
        self.slugs_after_item = []
        self.path = ''
        self.language = None
        self.node = None
        self.page = None
        self.item_type = ParsedPath.UNKNOWN

    def __str__(self):
        return 'ParsedPath(path=%s, slugs_in_path=%s, slugs_after_item=%s, '\
            'item_type=%s, node=%s, page=%s)' % (self.path, self.slugs_in_path, 
            self.slugs_after_item, self.ITEM_TYPES[self.item_type], 
            self.node, self.page)

# ============================================================================
# Site Management
# ============================================================================

class Site(TimeTrackedModel):
    """A Site object is the highest level container in the CMS.  It groups
    together pages, menus and associated configuration.
    """
    name = models.CharField(max_length=25, unique=True)
    domain = models.CharField(max_length=100, unique=True)
    doc_root = models.ForeignKey('yacon.Node', blank=True, null=True, 
        related_name='+')
    menus = models.ManyToManyField('yacon.Node', blank=True, null=True,
        related_name='+')
    default_language = models.ForeignKey(Language, related_name='+')
    alternate_language = models.ManyToManyField(Language, related_name='+')

    class Meta:
        app_label = 'yacon'

    # --------------------------------------------
    # Factory/Fetch Methods
    @classmethod
    def create_site(cls, name, domain, languages=[], config={}):
        """Creates a Site object with corresponding SiteURL and SiteConfig
        entries.

        :param name -- name of site
        :param domain -- domain of site
        :param languages -- a list of Language objects that will be the
            allowed languages for this site.   First item in the list is
            treated as the default language for the site.  If no list is given
            then Languages.default_language() is used to determine the sites
            default language.
        :param config -- optional dictionary which is converted into
            name/value pairs as SiteConfig entries

        :returns: Site object
        """
        default_language = None
        if len(languages) == 0:
            default_language = Language.default_language()
        else:
            default_language = languages.pop(0)

        # create the Site object and a document root to go along with it
        site = Site(name=name, domain=domain, default_language=default_language)
        site.save()
        site.create_doc_root()

        # set any remaining languages
        for lang in languages:
            site.alternate_language.add(lang)

        # for any config passed in create the corresponding objects
        for name in config.keys():
            sc = SiteConfig(site=site, name=name, value=config[name])
            sc.save()

        return site

    @classmethod
    def get_site(cls, request):
        """Uses the domain information from an HttpRequest object to find the
        corresponding site or raises a 404 if no matching site found"""
        domain = request.META['HTTP_HOST']
        logger.debug('using domain "%s"' % domain)
        site = get_object_or_404(Site, domain=domain)
        logger.debug('found site %s (id=%d)' % (site.name, site.id))

        return site

    def create_doc_root(self):
        """Creates the document root Node for the site.  The default Language
        is used for this Node; as the Node itself contains '/' as the slug,
        the language is more or less moot.  The name of the node should only
        ever show up in admin areas
        """
        if self.doc_root != None:
            raise AttributeError('There can be only one Site.doc_root at ' +\
                'a time.')

        self.doc_root = Node.add_root(site=self)
        self.save()
        tx = NodeTranslation(node=self.doc_root, name='root', slug='/',
            language=self.default_language)
        tx.save()

    # --------------------------------------------
    # Config Management

    def add_config(self, name, value):
        """Adds a SiteConfig name/value pair to this site

        :param name -- name of config to add
        :param value -- value of config to hold
        """
        config = SiteConfig(site=self, name=name, value=value)
        config.save()

    def get_config(self, name):
        """Returns a list of values for the given config name

        :param name -- name of config items to retrieve

        :returns: list of values, or empty list if no matching names
        """
        configs = SiteConfig.objects.filter(site=self, name=name)
        values = configs.values_list('value', flat=True)

        return list(values)

    # --------------------------------------------
    # Multilingual Routines

    def get_languages(self, language_code=None):
        """Returns a list of Language objects supported by this site.

        :param language_code -- restricts languages returned to just those who
            have a matching language_code

        :returns: list of Language objects for the valid languages for this
            site.  List may be empty if there are no matching codes
        """
        langs = []

        df = self.default_language
        if language_code == None:
            langs.append(df)
            langs.extend(self.alternate_language.all())
        else:
            if df.identifier.lower().startswith(language_code.lower()):
                langs.append(df)

            langs.extend(self.alternate_language.filter(
                identifier__istartswith=language_code))

        return langs

    def set_default_language(self, language):
        """Sets the default language to "language".  If the default language
        has changed, the current default language is moved to the list of
        alternate languages."""
        if self.default_language != language:
            if self.alternate_language.filter(id=language.id).count() == 0:
                # language isn't in the alterante list, add it
                self.alternate_language.add(self.default_language)

            self.default_language = language
            self.save()

    def language_count(self):
        """Returns number of translations for this site."""
        return self.alternate_language.all().count() + 1

    def has_missing_languages(self):
        """Returns True if there are defined languages that aren't assigned to
        this site."""
        langs = Language.objects.all().count()
        return langs != self.language_count()

    # -----------------------------------------------------------------------
    # Search Methods

    def parse_path(self, path):
        """A path of slugs is used to uniquely identify a Node or page in the
        document hierarchy, this method walks a given path until it runs out
        of either tree or the path, generating a ParsedPath object along the
        way.  The ParsedPath object contains the list of slugs in the path,
        any slugs after the path (i.e. if we run out of tree) and a resulting
        Node or PageTranslation object that corresponds to the path.  If the
        path does not result in finding a Node or page then a ParsedPath
        object is still returned.  

        As consistency of path language is not required, each level of the
        path can in theory be aliased in other languages.  The language
        returned is technically the Language of the last portion of the path,
        i.e. of either the last Node or PageTranslation found.

        If a Node object has a default_page setting and the path indicates the
        Node, then both a Node and PageTranslation object are returned, but 
        the item_type will indicate a PageTranslation.

        :param path: path to walk to find a container Node 

        :returns: ParsedPath object.
        """
        node = self.doc_root
        parsed = ParsedPath()
        parsed.path = path

        if path == '/':
            # special case for the home page
            parsed.node = self.doc_root
            parsed.item_type == ParsedPath.NODE
            page = parsed.node.get_default_page(node.site.default_language)
            if page != None:
                # root has a default_page
                parsed.item_type = ParsedPath.PAGE
                parsed.page = page

            return parsed

        parts = MATCH_SLASH.split(path)
        found_end_node = False
        # step through the parts of the path looking for Node objects
        for part in parts:
            # if the path contained double slashes, ignore them
            if part == '':
                continue 

            if not found_end_node:
                # search for a matching page within this node
                page = Page.find(node, [part, ])
                if page != None:
                    parsed.page = page
                    parsed.language = page.language
                    parsed.item_type = ParsedPath.PAGE
                    found_end_node = True
                else:
                    # no matching pages, look for child nodes
                    kids = list(node.get_children())
                    nts = NodeTranslation.objects.filter(slug=part, 
                        node__in=kids)
                    if len(nts) > 0:
                        # found slugs -- there should only be one, but to play
                        # it safe we don't enforce that
                        if len(nts) > 1:
                            logger.warning(('Multiple slugs matched for '
                                'children of node=%s and slug=%s' % (node, 
                                part)))

                        parsed.slugs_in_path.append(part)
                        parsed.node = nts[0].node
                        parsed.item_type = ParsedPath.NODE
                        node = nts[0].node
                    else:
                        parsed.slugs_after_item.append(part)
                        found_end_node = True
            else:
                # already found leaf Node, all slugs after this get stored
                parsed.slugs_after_item.append(part)

        if parsed.item_type == ParsedPath.UNKNOWN:
            # didn't find a Node in the path, return what we've built
            return parsed

        # if you get here then we found a Node, check for pages and default
        # pages 
        if len(parsed.slugs_after_item) > 0:
            # there are still slugs left after we found the last Node, see if
            # there is a page for it
            page = Page.find(parsed.node, parsed.slugs_after_item)
            if page != None:
                parsed.page = page
                parsed.language = page.language
                parsed.item_type = ParsedPath.PAGE

        # either there were no slugs after we found the Node, or there
        # were and there was no page for it; so we're only a Node, check
        # for default page for this Node
        if parsed.item_type == ParsedPath.NODE:
            # find the language of the Node so we can check for a default page
            last = parsed.slugs_in_path[-1]
            parsed.language = parsed.node.language_of_slug(last)

            # get the default page
            page = parsed.node.get_default_page(parsed.language)
            if page != None:
                # Node has a default_page
                parsed.item_type = ParsedPath.PAGE
                parsed.page = page

        return parsed

    def find_page(self, uri):
        """Returns a Page object corresponding to the given URI.  The URI can
        either be a path to a Page or to a Node with a default MetaPage.  If
        it is a Node with a default MetaPage, the last slug in the URI is used
        to attempt to determine the language and the Page for the MetaPage in
        the given language is returned.  If neither a Page or a Node with a
        MetaPage are not found, then None is returned.  If a Node with a
        MetaPage is found but it doesn't have the required translation then
        None is returned.

        :param uri: URI corresponding to a path to a Page or Node with a 
            default page

        :returns: Page object or None
        """
        parsed = self.parse_path('/' + uri)
        if parsed.item_type == ParsedPath.PAGE:
            return parsed.page

        # If you get here, then either: 1) URI was invalid, 2) URI was for a
        # Node without a default_page
        return None
            

class SiteConfig(TimeTrackedModel):
    """A name/value pair object for configuring a site"""

    site = models.ForeignKey(Site)
    name = models.CharField(max_length=100)
    value = models.CharField(max_length=100)

    class Meta:
        app_label = 'yacon'



# yacon.views.settings_tab.py
# blame ctrudeau chr(64) arsensa.com
#
# Views for the Settings tab in Nexus
#

import logging, urllib

from django.http import HttpResponse

from yacon.decorators import superuser_required
from yacon.models.common import Language

logger = logging.getLogger(__name__)

# ============================================================================
# Settings Page Ajax Methods
# ============================================================================

@superuser_required
def add_language(request, name, identifier):
    name = urllib.unquote(name)
    identifier = urllib.unquote(identifier).lower()

    Language.factory(name, identifier)
    return HttpResponse()



# yacon.utils.py
import logging, json, inspect, urllib, os, locale, re, time, tarfile
from datetime import datetime
from itertools import islice, chain
from PIL import Image
from zipfile import ZipFile

from django.conf import settings
from django.http import HttpResponse, Http404

from yacon import conf

logger = logging.getLogger(__name__)
locale.setlocale(locale.LC_ALL, '')

# ============================================================================
# Utility Classes
# ============================================================================

# Enum
#
# borrowed and modified from:
#  http://tomforb.es
#      /using-python-metaclasses-to-make-awesome-django-model-field-choices

class Enum(object):
    """A tuple of tuples pattern of (id, string) is common in django for
    choices fields, etc.  This object inspects its own members (i.e. the
    inheritors) and produces the corresponding tuples.

    Example:

    class Colours(Enum):
        RED = 'r'
        BLUE = 'b', 'Blueish'

    >> Colours.RED
    'r'
    >> list(Colours)
    [('r', 'Red'), ('b', 'Blueish')]
    """
    class __metaclass__(type):
        def __init__(self, *args, **kwargs):
            self._data = []
            for name, value in inspect.getmembers(self):
                if not name.startswith('_') and not inspect.ismethod(value):
                    if isinstance(value, tuple) and len(value) > 1:
                        data = value
                    else:
                        pieces = [x.capitalize() for x in name.split('_')]
                        data = (value, ' '.join(pieces))
                    self._data.append(data)
                    setattr(self, name, data[0])

            self._hash = dict(self._data)

        def __iter__(self):
            for value, data in self._data:
                yield (value, data)

    @classmethod
    def get_value(self, key):
        return self._hash[key]


class JSONResponse(HttpResponse):
    def __init__(self, obj, **kwargs):
        extra_headers = kwargs.pop('extra_headers', {})
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(json.dumps(obj), **kwargs)

        for key, value in extra_headers.items():
            self[key] = value


# QuerySetChain
#
# borrowed and modified from:
#   http://stackoverflow.com/questions/431628/
#   by: http://stackoverflow.com/users/15770/akaihola
class QuerySetChain(object):
    """
    Chains multiple subquerysets (possibly of different models) and behaves as
    one queryset.  Supports minimal methods needed for use with
    django.core.paginator.

    Usage:
        q1 = Thing.objects.filter(foo)
        q2 = Stuff.objects.filter(bar)
        qsc = QuerySetChain(q1, q2)
    """

    def __init__(self, *subquerysets):
        self.querysets = subquerysets

    def count(self):
        """
        Performs a .count() for all subquerysets and returns the number of
        records as an integer.
        """
        return sum(qs.count() for qs in self.querysets)

    def _clone(self):
        "Returns a clone of this queryset chain"
        return self.__class__(*self.querysets)

    def _all(self):
        "Iterates records in all subquerysets"
        return chain(*self.querysets)

    def __getitem__(self, index):
        """
        Retrieves an item or slice from the chained set of results from all
        subquerysets.
        """
        if type(index) is slice:
            return list(islice(self._all(), index.start, index.stop, 
                index.step or 1))
        else:
            return islice(self._all(), index, index+1).next()


class SummarizedPage(object):
    """Holder for Page objects with ability to return portions of its content
    for displaying summaries."""

    def __init__(self, page, block_key, summary_size, block=None):
        self.page = page
        self.block = block
        self.block_key = block_key
        self.summary_size = summary_size

    @property 
    def summary(self):
        value = ''
        blocks = self.page.get_blocks(self.block_key)
        if len(blocks) != 0:
            value = self.summarize(blocks[0].content, self.summary_size)

        return value

    @classmethod
    def summarize(cls, text, size_limit):
        """Returns a substring no larger than size_limit, attempts to keep whole
        words by cutting the string at the last space."""
        # remove any html tags
        subtext = re.sub('<[^<]+?>', '', text)[:size_limit]
        if len(subtext) < size_limit:
            return subtext

        index = subtext.rfind(' ')
        if index == -1:
            return subtext

        subtext = subtext[:index]
        return subtext

# ============================================================================
# File Browser Tools
# ============================================================================

class FileSpec(object):
    """Abstracts concept of public and private files and where they are served
    from."""
    def __init__(self, node, prefix=None, node_is_file=False):
        self.node = node
        self.prefix = prefix
        self.node_is_file = node_is_file
        self.basename = None
        self.relative_filename = None
        self.extension = None
        self._parse_node()

    @classmethod
    def factory_from_path(cls, path, ensure_file=False):
        """Takes a path and attempts to build a FileSpec object out of it.
        Raises AttributeError if path does not conform or isn't in either the
        public or private filespace."""
        if not os.path.exists(path):
            raise AttributeError('no such path')
        if ensure_file and not os.path.isfile(path):
            raise AttributeError('path was not a file')

        is_file = not os.path.isdir(path)
        if path.startswith(settings.MEDIA_ROOT):
            root_len = len(settings.MEDIA_ROOT)
            return FileSpec('public:%s' % path[root_len:], node_is_file=is_file)
        elif path.startswith(conf.site.private_upload):
            root_len = len(conf.site.private_upload)
            return FileSpec('private:%s' % path[root_len:], 
                node_is_file=is_file)

        raise AttributeError('path was not in public or private file space')

    @classmethod
    def factory_from_url(cls, url, ensure_file=False):
        if url.startswith(settings.MEDIA_URL):
            # url may be a public node
            root_len = len(settings.MEDIA_URL)
            node = 'public:%s' % url[root_len:]
        elif conf.site.private_upload_url \
                and url.startswith(conf.site.private_upload_url):
            # url may be a private node
            root_len = len(conf.site.private_upload_url)
            node = 'private:%s' % url[root_len:]

        spec = FileSpec(node, node_is_file=True)
        if ensure_file and not os.path.exists(spec.full_filename):
            raise AttributeError('no such path')

        return spec

    def _parse_node(self):
        pieces = self.node.split(':')
        try:
            self.file_type = pieces[0]
            x = urllib.unquote(pieces[1])
            if x and x[0] == '/':
                x = x[1:]

            if self.node_is_file:
                self.relative_dir = os.path.dirname(x)
            else:
                self.relative_dir = x

            if self.prefix:
                self.relative_dir = os.path.join(self.prefix, self.relative_dir)

            if self.file_type == 'public':
                self.full_dir = os.path.join(settings.MEDIA_ROOT,
                    self.relative_dir)
            elif self.file_type == 'private' and conf.site.private_upload:
                self.full_dir = os.path.join(conf.site.private_upload, 
                    self.relative_dir)
            elif self.file_type == 'system':
                # special case for creating root level folders in the admin
                if self.relative_dir == 'public':
                    self.full_dir = settings.MEDIA_ROOT
                elif self.relative_dir == 'private':
                    self.full_dir = conf.site.private_upload
                else:
                    raise Http404('bad path for system type')
            else:
                raise Http404('bad node type')

            if self.node_is_file:
                self.set_filename(os.path.basename(x))
        except IndexError:
            raise Http404('bad node key')

    def set_filename(self, filename):
        self.basename = filename
        if '.' in filename:
            pieces = filename.split('.')

            if len(pieces) > 2 and (
                    pieces[-1] in conf.site.compress_extensions):
                self.extension = '.'.join(pieces[-2:])
            else: # only one extension 
                self.extension = pieces[-1]

        self.relative_filename = os.path.join(self.relative_dir, filename)
        self.full_filename = os.path.join(self.full_dir, self.basename)
        if os.path.isdir(self.full_filename):
            self.node_is_file = False
        else:
            self.node_is_file = True

    def allowed_for_user(self, user):
        if user.is_superuser:
            return True

        # user is normal user, verify spec against the user's id
        name = 'users/%s' % user.id
        if self.relative_dir.startswith(name):
            return True

        if self.relative_dir.startswith('groups/'):
            pieces = self.relative_dir.split('/')
            from yacon.models.groupsq import GroupOfGroups
            try:
                group = GroupOfGroups.objects.get(id=pieces[1])
                if group.has_user(user):
                    return True

            except GroupOfGroups.DoesNotExist:
                return False

        return False

    def get_thumbnail(self, config=None):
        if not self.node_is_file:
            return None

        try:
            if not config:
                config = conf.site.auto_thumbnails['default']

            thumb_path = os.path.abspath(os.path.join(self.full_dir, 
                conf.site.auto_thumbnails['dir']))
            thumb_path = os.path.abspath(os.path.join(thumb_path, config))
            thumb_file = os.path.abspath(os.path.join(thumb_path,
                self.basename))

            if not os.path.exists(thumb_file):
                self.make_thumbnail(conf.site.auto_thumbnails['dir'],
                    config, conf.site.auto_thumbnails['config'][config][0],
                    conf.site.auto_thumbnails['config'][config][1])

            return FileSpec.factory_from_path(thumb_file, ensure_file=True)
        except KeyError:
            logger.exception('thumbnail requested but not properly configured')
            return None
        except AttributeError:
            logger.exception('thumbnail requested turned into bad path')
            return None

    def make_thumbnail(self, thumb_dir, thumb_key, width, height):
        image_dir = os.path.realpath(os.path.join(self.full_dir,
            thumb_dir, thumb_key))
        image_name = os.path.join(image_dir, self.basename)
        try:
            os.makedirs(image_dir)
        except:
            pass # already exists, do nothing

        # use PIL to create the thumbnail
        im = Image.open(self.full_filename)
        im.thumbnail((width, height), Image.ANTIALIAS)
        im.save(image_name, 'png')

    def expand_file(self):
        if not self.is_archive:
            return

        if self.extension in conf.site.zip_extensions:
            with ZipFile(self.full_filename) as f:
                f.extractall(self.full_dir)
        else:
            # tar extension
            with tarfile.open(self.full_filename) as f:
                # unlike ZipFile, TarFile isn't safe
                for name in f.getnames():
                    if '..' in name or name[0] == '/':
                        logger.warn('Ignoring tar with unsafe files: %s', 
                            self.full_filename)
                        return

                f.extractall(self.full_dir)

    @property
    def is_private(self):
        return self.file_type == 'private'

    @property
    def is_image(self):
        return self.extension in conf.site.image_extensions

    @property
    def is_archive(self):
        return (self.extension in conf.site.zip_extensions or \
            self.extension in conf.site.tar_extensions)

    @property
    def title(self):
        if self.basename:
            return self.basename

        return os.path.basename(self.relative_dir)

    @property
    def results(self):
        return {
            'success':True,
            'filename':self.relative_filename,
        }

    @property
    def json_results(self):
        return json.dumps(self.results)

    @property
    def key(self):
        if self.relative_filename:
            return '%s:%s' % (self.file_type, self.relative_filename)

        return '%s:%s' % (self.file_type, self.relative_dir)

    @property
    def url(self):
        if self.file_type == 'public':
            if self.node_is_file:
                return settings.MEDIA_URL + self.relative_filename
            else:
                return settings.MEDIA_URL + self.relative_dir
        elif self.file_type == 'private' and conf.site.private_upload:
            if self.node_is_file:
                return conf.site.private_upload_url + self.relative_filename
            else:
                return conf.site.private_upload_url + self.relative_dir

        return ''

    @property
    def uncached_url(self):
        now = datetime.now()
        epoch = int(time.mktime(now.timetuple()))
        ms = now.microsecond
        url = '%s?c=%s.%s' % (self.url, epoch, ms)
        return url


# -------------------
# Dynatree Methods

def files_subtree(spec, depth_limit, expanded):
    """Returns a hash representation in dynatree format of the public or
    private upload directories."""
    file_hash = {
        'title':spec.title,
        'key':spec.key,
        'icon':'fatcow/folder.png',
        'isLazy':True,
    }

    if depth_limit == 0 and file_hash['key'] not in expanded:
        # reached as far as we're going to go, check for kids
        has_child_directories = False
        for x in sorted(os.listdir(spec.full_dir), cmp=locale.strcoll):
            if os.path.isdir(x):
                has_child_directories = True
                break

        if has_child_directories:
            file_hash['isLazy'] = True

        return file_hash

    # process any child directories
    children = []
    for x in sorted(os.listdir(spec.full_dir), cmp=locale.strcoll):
        dir_path = os.path.abspath(os.path.join(spec.full_dir, x))
        if os.path.isdir(dir_path):
            dl = depth_limit
            if dl != -1:
                dl = dl - 1

            # if the file_type is system, then convert to what is under it
            # (either public or private), otherwise just add to the parent's
            # key
            if spec.file_type != 'system':
                key = '%s:%s' % (spec.file_type, os.path.join(
                    spec.relative_dir, x))
            else:
                key = '%s:%s' % (spec.basename, x)

            spec2 = FileSpec(key)
            subtree = files_subtree(spec2, dl, expanded)
            children.append(subtree)

    if children:
        file_hash['children'] = children

    return file_hash


def build_filetree(expanded, restricted=None):
    """Returns a dynatree hash representation of our public and private file
    directory hierarchy."""
    spec = FileSpec('system:public')
    spec.set_filename('public')
    public_node = {
        'title': 'Public',
        'key': 'system:public',
        'expand': True,
        'icon': 'fatcow/folders_explorer.png',
    }
    if restricted:
        # restricted to a single user's folders
        spec = FileSpec('public:users/%s' % restricted)
        public_node['children'] = {
            'title': restricted,
            'key': 'public:users/%s' % restricted,
            'activate':True,
            'isLazy': True,
            'icon':'fatcow/folder.png',
        }
    else:
        public = files_subtree(spec, 2, expanded)
        if 'children' in public:
            public_node['children'] = public['children']
            public['children'][0]['activate'] = True

    tree = [public_node,]

    if conf.site.private_upload:
        spec = FileSpec('system:private')
        spec.set_filename('private')
        private_node = {
            'title': 'Private',
            'key': 'system:private',
            'expand': True,
            'icon': 'fatcow/folders_explorer.png',
        }

        if restricted:
            # restricted to a single user's folders
            spec = FileSpec('private:users/%s' % restricted)
            private_node['children'] = {
                'title': restricted,
                'key': 'private:users/%s' % restricted,
                'isLazy': True,
                'icon':'fatcow/folder.png',
            }
        else:
            private = files_subtree(spec, 2, expanded)
            if 'children' in private:
                private_node['children'] = private['children']
                private['children'][0]['activate'] = True

        tree.append(private_node)

    return tree

# ============================================================================
# Miscellaneous Methods
# ============================================================================

# get_user_attributes
# 
# borrowed and modified from: 
#   http://stackoverflow.com/questions/4241171/inspect-python-class-attributes

def get_user_attributes(obj, exclude_methods=True):
    """Returns a list of non-system attributes for an object.

    :param obj: object or class to inspect
    :param exclude_methods: [optional] do not include callable methods in the
        returned list, defaults to True
    
    :returns: list of non-system attributes of an object or class
    """
    base_attributes = dir(type('dummy', (object,), {}))
    attributes = dir(obj)
    results = []
    for attribute in attributes:
        try:
            if attribute in base_attributes \
                    or (exclude_methods and callable(getattr(obj, attribute))):
                continue
            results.append(attribute)
        except AttributeError:
            # some kinds of access cause problems, ignore them
            pass

    return results


def get_profile(user):
    try:
        attr = conf.custom.user_curator.profile_class.__name__.lower()
        return getattr(user, attr)
    except:
        pass

    return None


def get_system_text(lang_code, key):
    try:
        return conf.site.text[lang_code][key]
    except KeyError:
        # fall through to default handling
        pass

    try:
        return conf.TEXT[key]
    except KeyError:
        return 'Text Error for key %s' % key
